# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 数据库模块 ]

"""
from .awaken_db import AwakenDb
from .awaken_sql import AwakenSql
from ...baseic.common.encryption import encrypt_md5


DB: AwakenDb = AwakenDb()
""" 数据库实例 """

SQL: AwakenSql = AwakenSql()
""" SQL语句实例 """


# 创建 admin 用户
if DB.execute(SQL.User.get_user_id_and_password(user_number='admin')).get('total') == 0:
    DB.execute(SQL.User.creation(
        user_number='admin',
        password=encrypt_md5('admin'),
        user_name='Admin',
        user_portrait='https://q1.qlogo.cn/g?b=qq&nk=190848757&s=640',
    ))

# 创建 test 用户
if DB.execute(SQL.User.get_user_id_and_password(user_number='test')).get('total') == 0:
    DB.execute(SQL.User.creation(
        user_number='test',
        password=encrypt_md5('123456'),
        user_name='Test',
        user_portrait='https://q1.qlogo.cn/g?b=qq&nk=190848757&s=640',
    ))

DB.commit()



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
自定义异常 - 基础

"""
import re

from ..decorator.baseic import deco_singleton
from ..common.template_rendering import TemplateRendering


_TEMPLATE_RENDERING = TemplateRendering()
_ERROR_POPUP_TITLE = '异常提示'


# --------------------------------------------------------------------------
class AwakenBaseError(Exception):
    """ 
    [ 自定义异常基类 ] 
    
    """
    err_name: str
    bas_message: str | list
    out_messages: list


    # ----------------------------------------------------------------------
    def __init__(self, error_message: str | list = []):
        # 获取自定义异常类的名称
        self.err_name = re.findall(r'\[ (.*) \]', self.__doc__)[0]
        self.out_messages = [f'[ {self.err_name} ]', '']
        self.bas_message = error_message

        if isinstance(self.bas_message, str):
            error_message = [error_message]
        self.bas_message = error_message


    # ----------------------------------------------------------------------
    def __str__(self) -> str:
        if len(self.bas_message) > 0:
            for messages in self.bas_message:
                if messages != '':
                    messages = f'- {messages}'
                self.out_messages.append(messages)

        _TEMPLATE_RENDERING.render_print(
            title=_ERROR_POPUP_TITLE,
            source=self.out_messages,
            is_show_number=False
        )
        if len(self.bas_message) >= 1:
            return self.bas_message[0]
        else:
            return self.err_name


# --------------------------------------------------------------------------
@deco_singleton
class ErrorRecorder:
    """ 错误记录器
    @ 作者 : chenjiancheng
    @ 描述 : NULL
    
    """
    _error_warehouse: list


    # ----------------------------------------------------------------------
    def __init__(self) -> None:
        self._error_warehouse = []

    
    # ----------------------------------------------------------------------
    def get_error_number(self):
        return len(self._error_warehouse)


    # ----------------------------------------------------------------------
    def record(self, error_message: str | list):
        if isinstance(error_message, str):
            self._error_warehouse.append(error_message)
        if isinstance(error_message, list):
            self._error_warehouse.extend(error_message)


    # ----------------------------------------------------------------------
    def template_out(self):
        _TEMPLATE_RENDERING.render_print(
            title='错误记录回溯',
            source=self._error_warehouse
        )
        self._error_warehouse.clear()


# --------------------------------------------------------------------------
G_ERROR_RECORDER: ErrorRecorder = ErrorRecorder()
""" 错误记录器全局实例 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ SQL蓝图 - 用户相关 ]

"""
from typing import Any

from ..sqlite_statement_handle import G_SQLITE_STATEMENT_HANDLE
from ....baseic.keyword import G_KEYWORD


# --------------------------------------------------------------------------
class SqlUser(object):
    """
    [ SQL蓝图 - 用户相关 ]

    """


    # ----------------------------------------------------------------------
    @staticmethod
    def get_user_id_and_password(user_number: str) -> Any:
        """
        [ 通过用户号码获取用户ID与密码 ]

        ---
        参数
        - user_number : { str } - 用户账号。

        """
        selecter = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.User)
        selecter.get(G_KEYWORD.DataBase.UserFieldKey.UserId)
        selecter.get(G_KEYWORD.DataBase.UserFieldKey.Password)
        selecter.filter_equal(G_KEYWORD.DataBase.UserFieldKey.UserNumber, user_number)
        return selecter


    # ----------------------------------------------------------------------
    @staticmethod
    def get_user_form_token(user_token: str) -> Any:
        """
        [ 根据登录态 token 查询用户 ]

        ---
        参数
        - user_token : { str } - 用户登录态。

        """
        selecter = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.User)
        selecter.get(G_KEYWORD.DataBase.UserFieldKey.UserId)
        selecter.get(G_KEYWORD.DataBase.UserFieldKey.UserNumber)
        selecter.get(G_KEYWORD.DataBase.UserFieldKey.UserName)
        selecter.get(G_KEYWORD.DataBase.UserFieldKey.Portrait)
        selecter.filter_equal(G_KEYWORD.DataBase.UserFieldKey.UserToken, user_token)
        return selecter


    # ----------------------------------------------------------------------
    @staticmethod
    def set_user_token(
        uid: int,
        token: str
    ) -> Any:
        """
        [ 更新任务 ]

        ---
        参数
        - uid : { int } - 用户ID。
        - token : { str } - 用户TOKEN。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.update(G_KEYWORD.DataBase.TableName.User)
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.UserToken, token)
        sql_handler.filter_equal(G_KEYWORD.DataBase.UserFieldKey.UserId, uid)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def creation(
        user_number: str,
        password: str,
        user_name: str,
        user_portrait: str = 'NULL',
    ) -> Any:
        """
        [ 创建用户 ]

        ---
        参数
        - user_number   : { str } - 用户账号。
        - password      : { str } - 用户密码。
        - user_name     : { str } - 用户名称。
        - user_portrait : { str } - 用户头像。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.insert(G_KEYWORD.DataBase.TableName.User)
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.UserId, 'NULL')
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.UserNumber, user_number)
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.Password, password)
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.UserName, user_name)
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.Portrait, user_portrait)
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.RoleId, 'NULL')
        sql_handler.push(G_KEYWORD.DataBase.UserFieldKey.UserToken, 'NULL')
        return sql_handler



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 用例记录相关接口 ]

"""
from flask import Response
from flask import Blueprint

from ..server_request_handler import URL_PREFIX
from ..server_request_handler import ServerRequestHandler
from ...db import DB
from ...db import SQL
from ....baseic.keyword import G_KEYWORD
from ....baseic.decorator.web_api import deco_webapi_error_capture


blueprint_record_case = Blueprint('record_case', __name__)
""" 用例记录接口蓝图 """


POST_RECORD_CASE_GET_COUNT      = '/record/case/count'
""" 接口URL - 获取用例记录数量 """
POST_RECORD_CASE_GET_LIST       = '/record/case/list'
""" 接口URL - 获取用例记录列表 """
POST_RECORD_CASE_GET_SPEND_TIME = '/record/case/spend_time'
""" 接口URL - 获取用例记录的消耗时间 """


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_record_case.route(''.join([URL_PREFIX, POST_RECORD_CASE_GET_COUNT]), methods=['POST'])
def post_count() -> Response:
    """
    [ 获取用例记录数量 ]

    ---
    可选参数
    - crid          : { str } - 用例记录编号。
    - trid          : { str } - 任务记录编号。
    - name          : { str } - 用例记录名称。
    - type          : { str } - 用例记录类型。
    - created_date  : { str } - 用例记录创建日期, 例如 2022-02-02。
    - state         : { str } - 用例记录状态。
    - start_time    : { str } - 用例记录开始时间。
    - end_time      : { str } - 用例记录结束时间。

    """
    result = {}
    
    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        optional_keys=[
            G_KEYWORD.Api.CaseRecordFieldKey.CRid,
            G_KEYWORD.Api.CaseRecordFieldKey.TRid,
            G_KEYWORD.Api.CaseRecordFieldKey.Name,
            G_KEYWORD.Api.CaseRecordFieldKey.Type,
            G_KEYWORD.Api.CaseRecordFieldKey.CreatedDate,
            G_KEYWORD.Api.CaseRecordFieldKey.State,
            G_KEYWORD.Api.CaseRecordFieldKey.StartTime,
            G_KEYWORD.Api.CaseRecordFieldKey.EndTime,
        ]
    )

    # 从数据库中获取数据
    db_date = DB.execute(SQL.CaseRecord.get_count(paras), return_data_type='LIST')
    count = db_date.get(G_KEYWORD.Common.TransactionFieldKey.Items)[0][0]

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Count: count
    })
    return ServerRequestHandler.successful(result)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_record_case.route(''.join([URL_PREFIX, POST_RECORD_CASE_GET_LIST]), methods=['POST'])
def post_list() -> Response:
    """
    [ 获取用例列表 ]

    ---
    可选参数
    - crid         : { str } - 用例记录编号。
    - trid         : { str } - 任务记录编号。
    - name         : { str } - 用例记录名称。
    - type         : { str } - 用例记录类型。
    - created_date : { str } - 用例记录创建日期, 例如 2022-02-02。
    - state        : { str } - 用例记录状态。
    - start_time   : { str } - 用例记录开始时间。
    - end_time     : { str } - 用例记录结束时间。
    - page         : { str } - 当前页数。
    - number       : { str } - 单页数据量。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        optional_keys=[
            G_KEYWORD.Api.CaseRecordFieldKey.CRid,
            G_KEYWORD.Api.CaseRecordFieldKey.TRid,
            G_KEYWORD.Api.CaseRecordFieldKey.Name,
            G_KEYWORD.Api.CaseRecordFieldKey.TRid,
            G_KEYWORD.Api.CaseRecordFieldKey.CreatedDate,
            G_KEYWORD.Api.CaseRecordFieldKey.State,
            G_KEYWORD.Api.CaseRecordFieldKey.StartTime,
            G_KEYWORD.Api.CaseRecordFieldKey.EndTime,
            G_KEYWORD.Common.PagingDataFieldKey.Page,
            G_KEYWORD.Common.PagingDataFieldKey.Number,
        ]
    )

    # 从数据库中获取数据
    db_data = DB.execute(SQL.CaseRecord.get_list(paras))
    items = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Items)
    totai = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Totai)

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Items: items,
        G_KEYWORD.Api.TransactionFieldKey.Totai: totai,
    })
    return ServerRequestHandler.successful(result)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_record_case.route(''.join([URL_PREFIX, POST_RECORD_CASE_GET_SPEND_TIME]), methods=['POST'])
def post_spend_time() -> Response:
    """
    [ 查询用例记录的消耗时间 ]

    ---
    可选参数
    - crid          : { str } - 用例记录编号。
    - trid          : { str } - 任务记录编号。
    - name          : { str } - 用例记录名称。
    - type          : { str } - 用例记录类型。
    - created_date  : { str } - 用例记录创建日期, 例如 2022-02-02。
    - result        : { str } - 用例记录结果。
    - start_time    : { str } - 用例记录开始时间。
    - end_time      : { str } - 用例记录结束时间。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        optional_keys=[
            G_KEYWORD.Api.CaseRecordFieldKey.CRid,
            G_KEYWORD.Api.CaseRecordFieldKey.TRid,
            G_KEYWORD.Api.CaseRecordFieldKey.Name,
            G_KEYWORD.Api.CaseRecordFieldKey.TRid,
            G_KEYWORD.Api.CaseRecordFieldKey.CreatedDate,
            G_KEYWORD.Api.CaseRecordFieldKey.Result,
            G_KEYWORD.Api.CaseRecordFieldKey.StartTime,
            G_KEYWORD.Api.CaseRecordFieldKey.EndTime,
        ]
    )

    # 从数据库中获取数据
    db_data  = DB.execute(SQL.CaseRecord.get_spend_time(paras))
    time_sum = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Items)[0]['sum']

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Time: time_sum
    })
    return ServerRequestHandler.successful(result)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 数据处理相关接口 ]

"""
import requests
from flask import Response
from flask import Blueprint

from ..server_request_handler import URL_PREFIX
from ..server_request_handler import ServerRequestHandler
from ....baseic.keyword import G_KEYWORD
from ....baseic.decorator.web_api import deco_webapi_error_capture


blueprint_processing = Blueprint('processing', __name__)
""" 数据处理接口蓝图 """


POST_PROCESSING_GET_URL_DOM = '/processing/get_url_dom'
""" 接口URL - 获取目标URL的DOM文档 """


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_processing.route(''.join([URL_PREFIX, POST_PROCESSING_GET_URL_DOM]), methods=['POST'])
def post_get_url_dom() -> Response:
    """
    [ POST - 获取目标URL的DOM文档 ]

    ---
    必要参数
    - url : { str } - 目标URL。

    """
    try:
        result = {}

        # 解析请求
        paras = ServerRequestHandler.analysis_request_parameter(
            must_keys=[
                G_KEYWORD.Api.ProcessingFieldKey.Url
            ]
        )
        url = paras.get(G_KEYWORD.Api.ProcessingFieldKey.Url)

        r = requests.get(url)
        r.encoding = 'UTF-8'
        document = r.text

        # 处理返回
        result.update({
            G_KEYWORD.Api.TransactionFieldKey.Data: document
        })
        return ServerRequestHandler.successful(result)

    except BaseException as error:
        return ServerRequestHandler.unsuccessful(str(error))



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ SQL蓝图 - 任务相关 ]

"""
import json
from typing import Any

from ..sqlite_statement_handle import G_SQLITE_STATEMENT_HANDLE
from ....baseic.const import G_CONST
from ....baseic.keyword import G_KEYWORD


# --------------------------------------------------------------------------
class SqlTasks:
    """
    [ SQL蓝图 - 任务相关 ]

    """


    # ----------------------------------------------------------------------
    @staticmethod
    def get_count(paras: dict = {}) -> Any:
        """
        [ 查询任务数量 ]

        ---
        可选参数
        - tid           : { str } - 任务编号。
        - pid           : { str } - 项目编号。
        - type          : { str } - 任务类型。
        - name          : { str } - 任务名称。
        - state         : { str } - 任务状态。
        - created_date  : { str } - 任务创建日期, 例如 2022-02-02。
        - run_number    : { str } - 运行次数。
        - last_run_date : { str } - 最后运行日期。

        """
        selecter = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.Tasks)
        selecter.get(G_KEYWORD.DataBase.TasksFieldKey.Tid, G_KEYWORD.Sqlite.SelectEvent.Count)
        selecter.filter_equal_batch(paras)
        return selecter


    # ----------------------------------------------------------------------
    @staticmethod
    def get_list(paras: dict = {}) -> Any:
        """
        [ 查询任务列表 ]

        ---
        可选参数
        - tid           : { str } - 任务编号。
        - pid           : { str } - 项目编号。
        - type          : { str } - 任务类型。
        - name          : { str } - 任务名称。
        - state         : { str } - 任务状态。
        - created_date  : { str } - 任务创建日期, 例如 2022-02-02。
        - run_number    : { str } - 运行次数。
        - last_run_date : { str } - 最后运行日期。
        - page          : { str } - 当前页数。
        - number        : { str } - 单页数据量。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.filter_equal_batch(paras)
        sql_handler.set_sort_desc(G_KEYWORD.DataBase.TasksFieldKey.Tid)
        page   = paras.get(G_KEYWORD.Common.PagingDataFieldKey.Page)
        number = paras.get(G_KEYWORD.Common.PagingDataFieldKey.Number)
        sql_handler.set_data_limit(page, number)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def get_task_script(tid: int) -> Any:
        """
        [ 查询任务脚本 ]

        ---
        必要参数
        - tid : { str } - 任务编号。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.get(G_KEYWORD.DataBase.TasksFieldKey.Type)
        sql_handler.get(G_KEYWORD.DataBase.TasksFieldKey.Name)
        sql_handler.get(G_KEYWORD.DataBase.TasksFieldKey.Docs)
        sql_handler.get(G_KEYWORD.DataBase.TasksFieldKey.ScriptJson)
        sql_handler.filter_equal(G_KEYWORD.DataBase.TasksFieldKey.Tid, tid)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def get_exist_task_dates(paras: dict = {}) -> Any:
        """
        [ 查询去重后的所有存在任务的日期列表 ]

        ---
        可选参数
        - tid           : { str } - 任务编号。
        - pid           : { str } - 项目编号。
        - name          : { str } - 任务名称。
        - type          : { str } - 任务类型。
        - created_date  : { str } - 任务创建日期, 例如 2022-02-02。
        - state         : { str } - 任务状态。
        - last_run_date : { str } - 最后运行日期。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.filter_equal_batch(paras)
        sql_handler.get(G_KEYWORD.DataBase.TasksFieldKey.CreatedDate, G_KEYWORD.Sqlite.SelectEvent.Distinct)
        sql_handler.set_sort_desc(G_KEYWORD.DataBase.TasksFieldKey.Tid)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def creation(
        pid: int,
        type: str,
        name: str,
        docs: str,
        created_date: str,
        state: str = G_CONST.State.Result.Null
    ) -> Any:
        """
        [ 任务创建 ]

        ---
        参数:
        - pid          : { int } - 任务编号。
        - type         : { int } - 任务类型。
        - name         : { int } - 任务名称。
        - docs         : { str } - 任务说明。
        - created_date : { str } - 任务创建日期, 例如 2022-02-02。
        - state        : { str } - 任务状态。

        """
        # 初始脚本JSON
        script_json = {
            G_KEYWORD.Script.ScriptJson.Type     : type,
            G_KEYWORD.Script.ScriptJson.Name     : name,
            G_KEYWORD.Script.ScriptJson.Docs     : docs,
            G_KEYWORD.Script.ScriptJson.Namespace: [],
            G_KEYWORD.Script.ScriptJson.Cases    : [],
        }
        script_json_string = json.dumps(script_json, ensure_ascii=False)

        sql_handler = G_SQLITE_STATEMENT_HANDLE.insert(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.deduction([
            'NULL', pid, type, name, docs, state, created_date, script_json_string, 0, 'NULL', 'NULL', 0
        ])
        return sql_handler


    # --------------------------------------------------------------------------
    @staticmethod
    def modify(
        tid: int,
        name: str,
        docs: str,
        script_json_string: str,
    ) -> Any:
        """
        [ 任务编辑 ]

        ---
        参数
        - tid                : { int } - 任务编号。
        - name               : { int } - 任务名称。
        - docs               : { str } - 任务说明。
        - script_json_string : { Any } - 脚本JSON字符串。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.update(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.push(G_KEYWORD.DataBase.TasksFieldKey.Name, name)
        sql_handler.push(G_KEYWORD.DataBase.TasksFieldKey.Docs, docs)
        sql_handler.push(G_KEYWORD.DataBase.TasksFieldKey.ScriptJson, script_json_string)
        sql_handler.filter_equal(G_KEYWORD.DataBase.TasksFieldKey.Tid, tid)
        return sql_handler


    # --------------------------------------------------------------------------
    @staticmethod
    def set_state(
        tid: int,
        state: str,
    ) -> Any:
        """
        [ 设置状态 ]

        ---
        参数
        - tid   : { int } - 任务编号。
        - state : { str } - 任务状态。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.update(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.push(G_KEYWORD.DataBase.TasksFieldKey.State, state)
        sql_handler.filter_equal(G_KEYWORD.DataBase.TasksFieldKey.Tid, tid)
        return sql_handler


    # --------------------------------------------------------------------------
    @staticmethod
    def delete(tid: int) -> Any:
        """
        [ 删除任务 ]

        ---
        参数
        - tid : { int } - 任务编号。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.delete(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.filter_equal(G_KEYWORD.DataBase.TasksFieldKey.Tid, tid)
        return sql_handler


    # --------------------------------------------------------------------------
    @staticmethod
    def delete_form_project(pid: int) -> Any:
        """
        [ 删除指定项目的任务 ]

        ---
        参数
        - pid : { int } - 任务编号。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.delete(G_KEYWORD.DataBase.TableName.Tasks)
        sql_handler.filter_equal(G_KEYWORD.DataBase.TasksFieldKey.Pid, pid)
        return sql_handler



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ AwakenApp ]

"""
from flask import Flask
from flask.blueprints import Blueprint

from ..api.blueprint import __all__ as blueprint_all
from ...baseic.keyword import G_KEYWORD
from ...baseic.broadcast.awaken_log import G_LOG
from ...baseic.common.configure import G_CONFIGURE
from ...baseic.decorator.baseic import deco_singleton


# --------------------------------------------------------------------------
@deco_singleton
class AwakenApp:
    """
    [ AwakenApp ]

    ---
    描述
    - 基于 Flask 实现的 Flask.WebApi 应用程序。

    """
    _app_living: Flask = Flask(__name__)
    """ 应用程序实例 """


    # ----------------------------------------------------------------------
    def __init__(self):
        self.update_config({'JSON_AS_ASCII': False})
        for blueprint in blueprint_all:
            self.load_blueprint(blueprint)


    # ----------------------------------------------------------------------
    @property
    def living(self) -> Flask:
        """ 
        [ 应用程序实例 ]

        ---
        返回
        - Flask : 返回应用程序实例。

        """
        return self._app_living


    # ----------------------------------------------------------------------
    def update_config(self, config_dict: dict) -> ...:
        """ 
        [ 更新配置项 ]

        ---
        描述
        - 更新 Flask 应用程序的配置项。

        ---
        参数
        - config_dict : { dict } - 配置项字典。

        """
        self._app_living.config.update(config_dict)

        if G_CONFIGURE.get(G_KEYWORD.Common.Config.Debug):
            for ck, cv in config_dict.items():
                G_LOG.info(f'{ self.__class__.__name__ } :: 更新配置 :: [{ ck } = { cv }]')


    # ----------------------------------------------------------------------
    def load_blueprint(self, blueprint: Blueprint) -> ...:
        """ 
        [ 载入蓝图 ]

        ---
        描述
        - 将蓝图对象载入 Flask 应用程序。

        ---
        参数
        - blueprint : { Blueprint } - Flask.Blueprint 蓝图对象。

        """
        self._app_living.register_blueprint(blueprint)

        if G_CONFIGURE.get(G_KEYWORD.Common.Config.Debug):
            G_LOG.info(f'{ self.__class__.__name__ } :: 载入蓝图 :: [{ blueprint }]')



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
命令行指令 - 帮助   

"""
from ..baseic.common.template_rendering import G_TEMPLATE_RENDERING


# --------------------------------------------------------------------------
def instruction_help():
    """
    [ 命令行指令 - 帮助 ]

    """
    G_TEMPLATE_RENDERING.render_print(
        title='Awaken - 帮助',
        is_show_number=False,
        source=[
            '暂未搭建帮助中心数据集合 !',
        ],
    )
    exit(0)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
命令行指令 - 运行任务

"""
from pathlib import Path

from ..baseic.const import G_CONST
from ..baseic.common.template_rendering import G_TEMPLATE_RENDERING


# --------------------------------------------------------------------------
def instruction_running_task(argv: list):
    """
    [ 命令行指令 - 运行任务 ]
                                                                          
    ---
    参数:
    - argv : { list } - 参数列表。

    """
    if not G_CONST.Path.FilePath.EngineeringInit.exists():
        G_TEMPLATE_RENDERING.render_print(
            title='Awaken - 运行任务',
            is_show_number=False,
            source=[
                '当前运行路径不是工程根目录;',
                '请在工程根目录执行该命令.'
            ],
        )
        exit(0)

    task_list = []
    def recursive_search_tasks(flist: list):
        for f in flist:
            if f.is_dir():
                nlist = list(f.glob('*'))
                recursive_search_tasks(nlist)
            else:
                if f.suffix[1:-4] == 'awaken':
                    task_list.append(f)

    if len(argv) >= 1:
        # 循环解析任务参数
        for task_arg in argv:
            if task_arg in ['.', '/', './', 'all', 'ALL']:
                task_arg_path = G_CONST.Path.CONSOLE_CWD
                task_projects = list(task_arg_path.glob('*'))
                recursive_search_tasks(task_projects)

            else:
                task_arg_path = Path(task_arg).resolve()
                # 如果参数路径指向目录
                if task_arg_path.is_dir():
                    task_projects = list(task_arg_path.glob('*'))
                    recursive_search_tasks(task_projects)

                # 如果参数路径指向文件
                else:
                    if task_arg_path.exists():
                        task_list.append(task_arg_path)


        if not G_CONST.Path.FilePath.EngineeringInit.exists():
            G_TEMPLATE_RENDERING.render_print(
                title='Awaken - 运行任务',
                is_show_number=False,
                source=[
                    '暂无任务可供执行 !',
                ],
            )
            exit(0)

        G_TEMPLATE_RENDERING.render_print(
            title='Awaken - 运行任务',
            is_show_number=False,
            source=[
                '多进程执行器正在消耗任务 ...',
                '等待执行的任务:',
                *[tn.name for tn in task_list]
            ],
        )

        from ..core.perform.perform_pool import PerFormPool
        from ..core.interpreter.grammar_parser import GrammarParser
        grammar_parser = GrammarParser()
        perform_pool = PerFormPool()
        for task in task_list:
            task = grammar_parser.parsing_file(task)
            perform_pool.put_task(task)
        perform_pool.running()
    
    else:
        G_TEMPLATE_RENDERING.render_print(
            title='Awaken - 运行任务',
            is_show_number=False,
            source=[
                '运行任务指令参数异常 !',
                '示例:',
                '>> awaken -run task1',
                '>> awaken -run task1 task2',
                '>> awaken -run project1',
                '>> awaken -run .',
            ],
        )
        exit(0)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 引擎套件通用 ]

"""
from ..baseic.const import G_CONST
from .web.web_engine_method_map import WEB_ENGINE_FUNCTION_MAP


WEB_TASK_TEMPLATE = \
"""
## 自述
TaskType = '#TASK_TYPE#'
TaskName = '#TASK_NAME#'
TaskDocs = ''

## 用例
CASE :: TestName :: Docs
    Goto >> 'http://www.baidu.com/'
    Send >> 'Awaken2' >> '#kw'
    ?? True
"""


ENGINE_TYPE_FUNCTION_MAP = {
    G_CONST.Type.Task.Web: WEB_ENGINE_FUNCTION_MAP,
}
""" [ 脚本后缀与引擎类型的映射字典 ]
@ 描述 : 提供给解释器用以判断引擎的合法方法。
"""

TASK_TEMPLATE_MAP = {
    G_CONST.Type.Task.Web : WEB_TASK_TEMPLATE
}
""" [ 脚本后缀与任务模板的映射字典 ]
@ 描述 : NULL
"""



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
[ 基础装饰器 ]

"""


# --------------------------------------------------------------------------
def deco_singleton(cls):
    """
    [ 单例模式装饰器 ]

    ---
    描述
    - 冠以该装饰器的类在整个程序的生命周期中只会被实例化一次, 重复实例化返回已有实例。

    """
    global_instances = {}
    def _deco_singleton(*args, **kwargs):
        if cls not in global_instances:
            global_instances[cls] = cls(*args, **kwargs)
        return global_instances[cls]
    return _deco_singleton



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
解释器结构体

"""
from ..const import G_CONST
from ..common.convert import try_converting_string_format
from ..error.interpreter import AwakenInterpreterParseException


class AwakenTranslation: ...
class AwakenTask: ...
class AwakenCase: ...
class AwakenCodeLine: ...


# --------------------------------------------------------------------------
class AwakenTranslation:
    """
    [ Awaken译文对象 ]

    ---
    描述
    - 该对象描述从 Awaken 语句集中解析翻译而来的底层译文与其他信息。

    """
    type: str
    codelines: list


    # ----------------------------------------------------------------------
    def __init__(self, type: str, codeline: list) -> ...:
        self.type = type
        self.codelines = codeline


# --------------------------------------------------------------------------
class AwakenTask:
    """ 
    [ Awaken任务对象 ]

    ---
    描述
    - 该对象描述自动化任务的完整信息。

    """
    translation: AwakenTranslation
    is_encapsulated: bool
    is_write_database: bool
    database_task_id: int
    namespace: dict
    cases: dict[str, AwakenCase]
    task_type: str


    # ----------------------------------------------------------------------
    def __init__(
        self, 
        translation: AwakenTranslation,
        is_write_database: bool, 
        database_task_id: int,
    ) -> ...:
        self.translation = translation
        self.is_write_database = is_write_database
        self.database_task_id = database_task_id
        self.task_type = translation.type


# --------------------------------------------------------------------------
class AwakenCase:
    """ 
    [ Awaken用例对象 ]

    ---
    描述
    - 该对象描述自动化任务用例的相关信息。

    """
    number: int
    name: str
    docs: str
    decorator: dict
    namespace: dict
    steps: list[AwakenCodeLine]

    def __init__(self, number, name, docs='', *_) -> ...:
        self.number = number
        self.name = name
        self.docs = docs
        self.decorator = {}
        self.namespace = {}
        self.steps = []


# --------------------------------------------------------------------------
class AwakenCodeLine:
    """ 
    [ Awaken代码行对象 ]

    ---
    描述
    - 该对象描述代码行的相关信息。

    """
    base_code: str
    number: int
    region: str
    type: str
    give_region: str
    give_name: str
    give_value: str
    funtion_region: str
    funtion_name: str
    funtion_value: str
    case_name: str
    case_docs: str
    decorator_key: str
    decorator_value: str
    assert_value: str


    # ----------------------------------------------------------------------
    def __init__(self, codeline: str) -> ...:
        self.base_code   = codeline
        codeline_splint = codeline.split(' ')
        self.number     = codeline_splint[0]
        self.region     = codeline_splint[1]
        self.type       = codeline_splint[2]

        if self.type == G_CONST.Interpreter.CodeLineType.Give:
            _give_splint     = codeline_splint[3].split(G_CONST.Interpreter.GrammarSymbol.ScopePrefix)
            self.give_region = _give_splint[0]
            self.give_name   = _give_splint[1]
            self.give_value  = try_converting_string_format(codeline_splint[4])

        elif self.type == G_CONST.Interpreter.CodeLineType.RGive:
            _give_splint        = codeline_splint[3].split(G_CONST.Interpreter.GrammarSymbol.ScopePrefix)
            _run_splint         = codeline_splint[4].split(G_CONST.Interpreter.GrammarSymbol.ScopePrefix)
            self.give_region    = _give_splint[0]
            self.give_name      = _give_splint[1]
            self.funtion_region = _run_splint[0]
            self.funtion_name   = _run_splint[1]
            self.funtion_value  = []
            if len(codeline_splint) >= 6:
                for value in codeline_splint[5:]:
                    self.funtion_value.append(try_converting_string_format(value))

        elif self.type == G_CONST.Interpreter.CodeLineType.Run:
            _run_splint         = codeline_splint[3].split(G_CONST.Interpreter.GrammarSymbol.ScopePrefix)
            self.funtion_region = _run_splint[0]
            self.funtion_name   = _run_splint[1]
            self.funtion_value  = []
            if len(codeline_splint) >= 5:
                for value in codeline_splint[4:]:
                    self.funtion_value.append(try_converting_string_format(value))

        elif self.type == G_CONST.Interpreter.CodeLineType.SCase:
            self.case_name = codeline_splint[3]
            self.case_docs = codeline_splint[4] if len(codeline_splint) == 5 else ''

        elif self.type == G_CONST.Interpreter.CodeLineType.SDecorator:
            self.decorator_key   = codeline_splint[2]
            self.decorator_value = codeline_splint[3]

        elif self.type == G_CONST.Interpreter.CodeLineType.Assert:
            self.assert_value = codeline_splint[3:]

        else:
            raise AwakenInterpreterParseException



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
服务端请求处理程序

"""
import json

from flask import jsonify
from flask import request
from flask import Response

from ...baseic.const import G_CONST
from ...baseic.broadcast.awaken_log import G_LOG
from ...baseic.error.server import AwakenWebApiServerError
from ...baseic.common.convert import try_converting_string_format


URL_PREFIX: str = '/api'
""" URL前缀 """


# --------------------------------------------------------------------------
class ServerRequestHandler:
    """
    [ 服务端请求处理程序 ]

    ---
    描述
    - 处理 WebApi 请求的相关事务, 如解析前端请求参数与返回响应对象。

    """


    # ----------------------------------------------------------------------
    @staticmethod
    def analysis_request_parameter(must_keys: list = [], optional_keys: list = []) -> dict:
        """ 
        [ 解析请求参数 ]

        ---
        描述
        - 解析请求中指定 Key, 如果该值为空, 则舍弃该 Key, 而不是返回 None;
        - 如果需要确保解析结果中必须携带指定 Key, 请通过 must_keys 定义。

        ---
        参数
        - must_keys : { list } - 必要的待解析 Keys, 不存在时抛出异常。
        - optional_keys : { list } - 可选的待解析 Keys, 不存在时会忽略。 

        ---
        返回
        - dict : 返回解析完成的参数字典。

        """
        request_handle = request.args
        if request.method == 'POST':
            try:
                request_handle = json.loads(request.data.decode('UTF-8'))
            except json.decoder.JSONDecodeError:
                request_handle = request.form

        return ServerRequestHandler._common_analysis_request_keys(request_handle, must_keys, optional_keys)


    # ----------------------------------------------------------------------
    @staticmethod
    def analysis_request_headers(must_keys: list = [], optional_keys: list = []) -> dict:
        """
        [ 解析请求头 ]

        ---
        参数
        - must_keys : { list } - 必要的待解析 Keys, 不存在时抛出异常。
        - optional_keys : { list } - 可选的待解析 Keys, 不存在时会忽略。 

        ---
        返回
        - dict : 返回解析完成的参数字典。
        
        """
        headers_handle = request.headers
        return ServerRequestHandler._common_analysis_request_keys(headers_handle, must_keys, optional_keys)


    # ----------------------------------------------------------------------
    @staticmethod
    def successful(result: dict, message=G_CONST.Api.StateMessage.Success) -> Response:
        """
        [ 请求成功 ]

        ---
        描述
        - 将数据序列化为 Json 并包装成 Flask.Response 响应体返回。

        ---
        参数
        - result : { dict } - 需要返回的数据。

        ---
        返回
        - Response : 响应体对象。
         
        """
        return jsonify({
            'code': G_CONST.Api.StateCode.Success, 
            'message': message,
            'result': result
        })


    # ----------------------------------------------------------------------
    @staticmethod
    def unsuccessful(error: str, message=G_CONST.Api.StateMessage.Error) -> Response:
        """
        [ 请求失败 ]

        ---
        描述
        - 将数据序列化为 Json 并包装成 Flask.Response 响应体返回。

        ---
        参数
        - error : { str } - 需要返回的错误信息。

        ---
        返回
        - Response : 响应体对象。
         
        """
        return jsonify({
            'code': G_CONST.Api.StateCode.Error, 
            'message': message,
            'error': error
        })


    # ----------------------------------------------------------------------
    @staticmethod
    def _common_analysis_request_keys(data: dict, must_keys: list, optional_keys: list) -> dict:
        """
        [ 通用的解析参数 ]

        ---
        描述
        - 解析请求参数与解析请求头的通用函数。

        ---
        参数
        - data : { dict } - 待解析的数据。
        - must_keys : { list } - 必要的待解析 Keys, 不存在时抛出异常。
        - optional_keys : { list } - 可选的待解析 Keys, 不存在时会忽略。 

        """
        decryption_parameter: dict = {}

        # 解析必要的 Keys
        for key in must_keys:
            value = data.get(key)
            if value == None:
                raise AwakenWebApiServerError(f'请求数据不存在必要参数 "{key}" !')
            else:
                if value != '':
                    value = try_converting_string_format(value)
                decryption_parameter.update({key: value})
        
        # 解析可选的 Keys
        for key in optional_keys:
            value = data.get(key)
            if value == None:
                ...
            else:
                if value != '':
                    value = try_converting_string_format(value)
                decryption_parameter.update({key: value})

        return decryption_parameter



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 接口模块蓝图 ]

"""
from .api_user import blueprint_user
from .api_interpreter import blueprint_interpreter
from .api_processing import blueprint_processing
from .api_project import blueprint_project
from .api_tasks import blueprint_tasks
from .api_editor import blueprint_editor
from .api_runer import blueprint_runer
from .api_record_task import blueprint_record_task
from .api_record_case import blueprint_record_case


__all__ = [
    blueprint_user,
    blueprint_interpreter,
    blueprint_processing,
    blueprint_project,
    blueprint_tasks,
    blueprint_editor,
    blueprint_runer,
    blueprint_record_task,
    blueprint_record_case,
]



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 引擎代理 ]

"""
from .web.web_runner import WebRunner
from ..baseic.const import G_CONST
from ..baseic.structural.interpreter import AwakenTask
from ..core.interpreter.task_preprocessor import TaskPreprocessor


# --------------------------------------------------------------------------
class EngineAgency:
    """
    [ 引擎代理 ]

    """
    task_preprocessor: TaskPreprocessor
    """ 任务预处理程序 """


    # ----------------------------------------------------------------------
    def __init__(self) -> None:
        self.task_preprocessor = TaskPreprocessor()


    # ----------------------------------------------------------------------
    def start(self, task: AwakenTask):
        task = self.task_preprocessor.pretreatment(task)

        if task.translation.type == G_CONST.Type.Task.Web:
            runner = WebRunner(task)
            runner.start()



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
数据库关键字
    
"""


# --------------------------------------------------------------------------
class TableName:
    """
    [ 表名关键字 ]

    """
    User = 'user'
    """ 用户表 """

    Role = 'role'
    """ 角色表 """
    
    Project = 'project'
    """ 项目表 """

    Tasks = 'tasks'
    """ 任务表 """

    TaskRecord = 'task_record'
    """ 任务记录表 """

    CaseRecord = 'case_record'
    """ 用例记录表 """


# --------------------------------------------------------------------------
class UserFieldKey:
    """
    [ 用户表字段名关键字 ]

    """
    UserId = 'user_id'
    """ 用户ID """

    UserNumber = 'user_number'
    """ 用户账号 """

    Password = 'password'
    """ 用户密码 """

    UserName = 'user_name'
    """ 用户名称 """

    Portrait = 'portrait'
    """ 用户头像 """

    RoleId = 'role_id'
    """ 用户角色ID """

    UserToken = 'user_token'
    """ 用户登录态 """


# --------------------------------------------------------------------------
class RoleFieldKey:
    """
    [ 角色表字段名关键字 ]

    """
    RoleId = 'role_id'
    """ 角色ID """

    RoleName = 'role_name'
    """ 角色名称 """

    RoleValue = 'role_value'
    """ 角色权限 """


# --------------------------------------------------------------------------
class ProjectFieldKey:
    """
    [ 项目表字段名关键字 ]

    """
    Pid = 'pid'
    """ 项目ID """

    Name = 'name'
    """ 项目名称 """

    Type = 'type'
    """ 项目类型 """

    Docs = 'docs'
    """ 项目说明 """

    State = 'state'
    """ 状态 """
    
    CreatedDate = 'created_date'
    """ 创建日期 """


# --------------------------------------------------------------------------
class TasksFieldKey:
    """
    [ 任务表字段名关键字 ]

    """
    Tid = 'tid'
    """ 任务ID """

    Pid = 'pid'
    """ 项目ID """

    Name = 'name'
    """ 名称 """

    Docs = 'docs'
    """ 说明 """

    Type = 'type'
    """ 类型 """

    CreatedDate = 'created_date'
    """ 创建日期 """

    State = 'state'
    """ 状态 """

    ScriptJson = 'script_json'
    """ 脚本Json """

    RunNumber = 'run_number'
    """ 运行次数 """

    LastRunDate = 'last_run_date'
    """ 最后运行日期 """

    LastRunTime = 'last_run_time'
    """ 最后运行时间 """

    SpendTime = 'spend_time'
    """ 消耗时间 """


# --------------------------------------------------------------------------
class TaskRecordFieldKey:
    """
    [ 任务记录字段名关键字 ]

    """
    TRid = 'trid'
    """ 编号 """

    Tid = 'tid'
    """ 所属任务ID """

    Name = 'name'
    """ 名称 """

    Docs = 'docs'
    """ 说明 """

    Type = 'type'
    """ 类型 """

    CreatedDate = 'created_date'
    """ 创建日期 """

    State = 'state'
    """ 状态 """

    StartTime = 'start_time'
    """ 开始时间 """

    EndTime = 'end_time'
    """ 结束时间 """

    SpendTime = 'spend_time'
    """ 消耗时间 """

    Process = 'process'
    """ 过程 """


# --------------------------------------------------------------------------
class CaseRecordFieldKey:
    """
    [ 用例记录字段名关键字 ]

    """
    CRid = 'crid'
    """ 编号 """

    TRid = 'trid'
    """ 所属任务记录ID """

    Name = 'name'
    """ 名称 """

    Docs = 'docs'
    """ 说明 """
    
    Type = 'type'
    """ 类型 """

    CreatedDate = 'created_date'
    """ 创建日期 """

    State = 'state'
    """ 状态 """

    StartTime = 'start_time'
    """ 开始时间 """

    EndTime = 'end_time'
    """ 结束时间 """

    SpendTime = 'spend_time'
    """ 消耗时间 """

    Process = 'process'
    """ 过程 """




# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 解释器相关接口 ]

"""
from flask import Response
from flask import Blueprint

from ..server_request_handler import URL_PREFIX
from ..server_request_handler import ServerRequestHandler
from ....baseic.decorator.web_api import deco_webapi_error_capture
from ....baseic.const import G_CONST
from ....baseic.keyword import G_KEYWORD
from ....kit.web.web_engine_method_map import WEB_INSTRUCTIONS


blueprint_interpreter = Blueprint('interpreter', __name__)
""" 解释器接口蓝图 """


POST_INTERPRETER_GET_INSTRUCTIONS = '/interpreter/instructions'
""" 接口URL - 获取指令集 """


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_interpreter.route(''.join([URL_PREFIX, POST_INTERPRETER_GET_INSTRUCTIONS]), methods=['POST'])
def post_instructions() -> Response:
    """
    [ POST - 获取指令集 ]

    ---
    必要参数:
    - task_type : { str } - 任务类型。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        must_keys=[G_KEYWORD.Api.InterpreterFieldKey.TaskType]
    )
    task_type = paras.get(G_KEYWORD.Api.InterpreterFieldKey.TaskType)

    # 获取任务类型的指令集
    instructions = []
    if task_type.upper() == G_CONST.Type.Task.Web:
        instructions = WEB_INSTRUCTIONS

    # 处理返回
    result.update({
        G_KEYWORD.Api.InterpreterFieldKey.Instructions: instructions
    })
    return ServerRequestHandler.successful(result)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
[ 单元测试装饰器 ]

"""
from ..broadcast.awaken_log import G_LOG


# --------------------------------------------------------------------------
def deco_test_capture_exception(cls):
    """ 
    [ 捕获运行异常装饰器 ]

    ---
    描述
    - 冠以该装饰器的函数在执行异常时返回断言, 用以单元测试。

    """
    def _deco_test_capture_exception(*args, **kwargs):
        try:
            cls(*args, **kwargs)
            assert True
        except BaseException as error:
            G_LOG.error(str(error))
            assert False
    return _deco_test_capture_exception



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
浏览器驱动管理器

"""
import os
import shutil

from pathlib import Path

from ...broadcast.awaken_log import G_LOG
from ...common.template_rendering import G_TEMPLATE_RENDERING
from ...decorator.baseic import deco_singleton
from ...error.file import AwakenUserConfigPathNotExist
from ...error.type_error import AwakenBrowserDriverNotSupportError


_REGISTERED_BROWSERS = [
    'chromium', 'firefox', 'webkit',
]
""" 支持的浏览器清单 """

_ENVIRONMENT_VARIABLE = 'USERPROFILE'
""" 用户配置文件系统环境变量KEY """

_PLAYWRIGHT_CACHE_PATH = Path(os.environ[_ENVIRONMENT_VARIABLE]).joinpath('AppData\Local\ms-playwright')
""" Playwright缓存目录路径对象 """


# --------------------------------------------------------------------------
@deco_singleton
class BrowserDriverHandle:
    """
    [ 浏览器驱动管理器 ]

    """
    local_driver_path_map = {}


    # ----------------------------------------------------------------------
    def __init__(self) -> None:
        self._search_local_downloaded_browser_drivers()


    # ----------------------------------------------------------------------
    def get_registered_browsers(self):
        """ 
        [ 获取支持的浏览器清单 ] 
        
        """
        return _REGISTERED_BROWSERS


    # ----------------------------------------------------------------------
    def inspect(self, browser_drivers: str | list = []):
        """
        [ 检查已安装的浏览器 ]

        ---
        描述
        - 检查本地是否安装浏览器驱动。

        ---
        参数
        - browser_drivers : { str | list } - 需要查询的浏览器驱动名称或列表, 为空表示全部。

        ---
        返回
        - dict : 查询的浏览器驱动是否安装结果字典。

        """
        # 检查本地 playwright 环境是否安装浏览器驱动
        self._search_local_downloaded_browser_drivers()
        browser_drivers = self._unified_processing_browser_driver_list(browser_drivers)

        drive_installation_results = {}
        for driver_item in browser_drivers:
            result = False
            if driver_item in self.local_driver_path_map.keys():
                result = True
            drive_installation_results.update({driver_item : result})
        return drive_installation_results


    # ----------------------------------------------------------------------
    def install(self, browser_drivers: str | list = []):
        """
        [ 安装浏览器驱动 ]

        ---
        描述
        - 安装指定的浏览器驱动。

        ---
        参数
        - browser_drivers : { str | list } - 需要安装的浏览器驱动名称或列表, 为空表示全部。

        ---
        返回
        - dict : 浏览器驱动的安装结果字典。

        """
        drive_install_results = {}
        drive_installation_results = self.inspect()
        browser_drivers = self._unified_processing_browser_driver_list(browser_drivers)

        for driver_item in browser_drivers:
            install_result = True
            if drive_installation_results[driver_item] == True:
                drive_install_results.update({driver_item : install_result})
                continue
            try:
                G_LOG.debug(f'正在连接网络下载 [{driver_item}] 浏览器驱动.')
                G_TEMPLATE_RENDERING.render_print(
                    title='浏览器驱动管理器',
                    source=[f'正在连接网络下载 [{driver_item}] 浏览器驱动.'],
                    is_show_number=False
                )
                os.system(f'playwright install { driver_item }')
            except BaseException:
                install_result = False
                continue
            drive_install_results.update({driver_item : install_result})

        return drive_install_results


    # ----------------------------------------------------------------------
    def uninstall(self, browser_drivers: str | list = []):
        """
        [ 卸载浏览器驱动 ]

        ---
        描述
        - 卸载指定的浏览器驱动。

        ---
        参数
        - browser_drivers : { str | list } - 需要卸载的浏览器驱动名称或列表, 为空表示全部。

        ---
        返回
        - dict : 浏览器驱动的卸载结果字典。

        """
        drive_uninstall_results = {}
        drive_installation_results = self.inspect()
        browser_drivers = self._unified_processing_browser_driver_list(browser_drivers)

        for driver_item in browser_drivers:
            uninstall_result = True
            if drive_installation_results[driver_item] == False:
                drive_uninstall_results.update({driver_item : uninstall_result})
                continue
            try:
                G_LOG.debug(f'正在连接网络下载 [{driver_item}] 浏览器驱动.')
                G_TEMPLATE_RENDERING.render_print(
                    title='浏览器驱动管理器',
                    source=[f'正在从本地删除 [{driver_item}] 浏览器驱动.'],
                    is_show_number=False
                )
                shutil.rmtree(self.local_driver_path_map[driver_item])
                del self.local_driver_path_map[driver_item]
            except BaseException:
                uninstall_result = False
                continue
            drive_uninstall_results.update({driver_item : uninstall_result})
    
        return drive_uninstall_results


    # ----------------------------------------------------------------------
    def _search_local_downloaded_browser_drivers(self):
        """
        [ 搜索本地下载的浏览器驱动 ]

        ---
        描述
        - 从本地的Playwright缓存目录中搜索下载的浏览器驱动。

        """
        if _ENVIRONMENT_VARIABLE in os.environ:
            for dir_path in _PLAYWRIGHT_CACHE_PATH.glob('*'):
                driver_name = dir_path.stem.split('-')[0]
                if driver_name in _REGISTERED_BROWSERS:
                    self.local_driver_path_map.update({driver_name : dir_path})
        else:
            error_message = '未能在系统环境变量中获取到用户配置文件路径 !'
            raise AwakenUserConfigPathNotExist(error_message)


    # ----------------------------------------------------------------------
    def _unified_processing_browser_driver_list(self, browser_drivers: str | list):
        """
        [ 统一处理浏览器驱动列表 ]

        ---
        参数
        - browser_drivers : { str | list } - 浏览器驱动名称或列表。

        """
        browser_drivers = [browser_drivers] if isinstance(browser_drivers, str) else browser_drivers
        
        if len(browser_drivers) < 1:
            browser_drivers = _REGISTERED_BROWSERS

        for driver_item in browser_drivers:
            if driver_item not in _REGISTERED_BROWSERS:
                error_message = f'{driver_item} 不是支持的浏览器类型 !'
                raise AwakenBrowserDriverNotSupportError(error_message)

        return browser_drivers


# --------------------------------------------------------------------------
G_BROWSER_DRIVER_HANDLE: BrowserDriverHandle = BrowserDriverHandle()
""" 浏览器驱动管理器全局实例 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ SQL蓝图 - 项目相关 ]

"""
from typing import Any

from ..sqlite_statement_handle import G_SQLITE_STATEMENT_HANDLE
from ....baseic.keyword import G_KEYWORD


# --------------------------------------------------------------------------
class SqlProject:
    """
    [ SQL蓝图 - 项目相关 ]

    """


    # ----------------------------------------------------------------------
    @staticmethod
    def get_count(paras: dict = {}) -> str:
        """
        [ 查询项目数量 ]

        ---
        可选参数:
        - pid          : { str } - 项目ID。
        - type         : { str } - 项目类型。
        - state        : { str } - 项目状态。
        - created_date : { str } - 项目创建日期, 例如 2022-02-02。

        """
        selecter = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.Project)
        selecter.get(G_KEYWORD.DataBase.ProjectFieldKey.Pid, G_KEYWORD.Sqlite.SelectEvent.Count)
        selecter.filter_equal_batch(paras)
        return selecter


    # ----------------------------------------------------------------------
    @staticmethod
    def get_list(paras: dict = {}) -> Any:
        """
        [ 查询项目列表 ]

        ---
        可选参数:
        - pid          : { str } - 项目ID。
        - type         : { str } - 项目类型。
        - state        : { str } - 项目状态。
        - created_date : { str } - 项目创建日期, 例如 2022-02-02。
        - page         : { str } - 当前页数。
        - number       : { str } - 单页数据量。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.Project)
        sql_handler.filter_equal_batch(paras)
        sql_handler.set_sort_desc(G_KEYWORD.DataBase.ProjectFieldKey.Pid)
        page   = paras.get(G_KEYWORD.Common.PagingDataFieldKey.Page)
        number = paras.get(G_KEYWORD.Common.PagingDataFieldKey.Number)
        sql_handler.set_data_limit(page, number)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def creation(
        type: str,
        name: str,
        docs: str,
        state: str,
        created_date: str,
    ) -> Any:
        """
        [ 项目创建 ]

        ---
        参数:
        - type         : { str } - 项目类型。
        - name         : { str } - 项目名称。
        - docs         : { str } - 项目说明。
        - state        : { str } - 项目状态。
        - created_date : { str } - 项目创建日期, 例如 2022-02-02。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.insert(G_KEYWORD.DataBase.TableName.Project)
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.Pid, 'NULL')
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.Type, type)
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.Name, name)
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.Docs, docs)
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.State, state)
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.CreatedDate, created_date)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def modify(
        pid: int,
        name: str,
        docs: str,
    ) -> Any:
        """
        [ 项目编辑 ]

        ---
        参数
        - pid  : { int } - 项目ID。
        - name : { str } - 项目名称。
        - docs : { str } - 项目说明。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.update(G_KEYWORD.DataBase.TableName.Project)
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.Name, name)
        sql_handler.push(G_KEYWORD.DataBase.ProjectFieldKey.Docs, docs)
        sql_handler.filter_equal(G_KEYWORD.DataBase.ProjectFieldKey.Pid, pid)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def delete(pid: int) -> Any:
        """
        [ 项目删除 ]

        ---
        参数:
        - pid : { str } - 项目ID。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.delete(G_KEYWORD.DataBase.TableName.Project)
        sql_handler.filter_equal(G_KEYWORD.DataBase.ProjectFieldKey.Pid, pid)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def delete_all() -> Any:
        """
        [ 删除全部项目 ]

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.delete(G_KEYWORD.DataBase.TableName.Project)
        return sql_handler



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
命令行指令 - 运行服务器

"""
from ..baseic.const import G_CONST
from ..baseic.common.template_rendering import G_TEMPLATE_RENDERING
from ..core.perform.perform_server import PerformServer


# --------------------------------------------------------------------------
def instruction_running_server(argv: list):
    """
    [ 命令行指令 - 运行服务器 ]

    ---
    参数
    - argv : { list } - 参数列表。

    """
    if not G_CONST.Path.FilePath.EngineeringInit.exists():
        G_TEMPLATE_RENDERING.render_print(
            title='Awaken - 运行服务器',
            is_show_number=False,
            source=[
                '当前运行路径不是工程根目录;',
                '请在工程根目录执行该命令.'
            ],
        )
        exit(0)

    port: int = 3700
    if len(argv) >= 1:
        try:
            port = int(argv[0])
        except ValueError:
            G_TEMPLATE_RENDERING.render_print(
                title='Awaken - 运行服务器',
                is_show_number=False,
                source=[
                    '运行服务器指令的端口参数必须为数字 !'
                ],
            )
            exit(0)

    G_TEMPLATE_RENDERING.render_print(
        title='Awaken - 运行服务器',
        is_show_number=False,
        source=[
            f'服务器正在运行 >> 端口号 :: [ {port} ] !'
        ],
    )
    perform_server = PerformServer()
    perform_server.running(port)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 用户相关接口 ]

"""
import time

from flask import Response
from flask import Blueprint

from ..server_request_handler import URL_PREFIX
from ..server_request_handler import ServerRequestHandler
from ...db import DB
from ...db import SQL
from ....baseic.keyword import G_KEYWORD
from ....baseic.common.encryption import encrypt_md5
from ....baseic.decorator.web_api import deco_webapi_error_capture


blueprint_user = Blueprint('user', __name__)
""" 用户接口蓝图 """


POST_USER_GET_USER_INFO = '/user/user_info'
""" 接口URL - 获取用户信息 """
POST_USER_LOGIN         = '/user/login'
""" 接口URL - 用户登入 """
POST_USER_LOGOUT        = '/user/logout'
""" 接口URL - 用户登出 """


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_user.route(''.join([URL_PREFIX, POST_USER_GET_USER_INFO]), methods=['POST'])
def get_user_info() -> Response:
    """
    [ POST - 获取用户信息 ]
    
    """
    result = {}

    # 解析请求头
    headers = ServerRequestHandler.analysis_request_headers(
        must_keys=['Authorization']
    )
    user_token = headers['Authorization']

    # 从数据库中获取数据
    db_data = DB.execute(SQL.User.get_user_form_token(user_token=user_token))
    result = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Items)[0]

    # 角色暂时先写死不实现
    result[G_KEYWORD.Api.UserFieldKey.Role] = {
        'role_name': 'Super Admin', 'value': 'super'
    }

    # 处理返回
    return ServerRequestHandler.successful(result)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_user.route(''.join([URL_PREFIX, POST_USER_LOGIN]), methods=['POST'])
def login() -> Response:
    """
    [ POST - 用户登入 ]
    
    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        must_keys=[
            G_KEYWORD.Api.UserFieldKey.UserNumber,
            G_KEYWORD.Api.UserFieldKey.Password,
        ]
    )
    para_user_number   = paras.get(G_KEYWORD.Api.UserFieldKey.UserNumber)
    para_user_password = paras.get(G_KEYWORD.Api.UserFieldKey.Password)

    # 访问数据库
    db_data = DB.execute(SQL.User.get_user_id_and_password(para_user_number))
    db_user_info = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Items)

    if len(db_user_info) == 0:
        error_type    = "登录失败"
        error_message = "用户名或密码错误, 请重试 !"
        return ServerRequestHandler.unsuccessful(error=error_type, message=error_message)

    # 将传递的密码加密后与数据库密码比对校验
    # 密码校验通过时
    # 生成用户登录态 token 并写入数据库
    db_user_id = db_user_info[0].get(G_KEYWORD.DataBase.UserFieldKey.UserId)
    db_user_encry_password = db_user_info[0].get(G_KEYWORD.DataBase.UserFieldKey.Password)
    para_user_encry_password = encrypt_md5(para_user_password)
    if para_user_encry_password == db_user_encry_password:
        material = [para_user_number, para_user_password, str(int(time.time()))]
        token = encrypt_md5(''.join(material))
        DB.execute(SQL.User.set_user_token(uid=db_user_id, token=token))
        DB.commit()

        # 处理返回
        result.update({
            G_KEYWORD.Api.UserFieldKey.UserId : db_user_id,
            G_KEYWORD.Api.UserFieldKey.Token  : token,
            G_KEYWORD.Api.UserFieldKey.Role   : {
                'role_name': 'Super Admin',
                'value'    : 'super'
            }
        })
        return ServerRequestHandler.successful(result)
    
    else:
        error_type    = "登录失败"
        error_message = "用户名或密码错误, 请重试 !"
        return ServerRequestHandler.unsuccessful(error=error_type, message=error_message)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_user.route(''.join([URL_PREFIX, POST_USER_LOGOUT]), methods=['POST'])
def logout() -> Response:
    """
    [ 退出登录 ]

    """
    result = {}

    # 解析请求头
    headers = ServerRequestHandler.analysis_request_headers(
        must_keys=['Authorization']
    )
    user_token = headers['Authorization']

    # 访问数据库
    db_data = DB.execute(SQL.User.get_user_form_token(user_token))
    db_user_info = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Items)[0]
    db_user_id   = db_user_info.get(G_KEYWORD.DataBase.UserFieldKey.UserId)
    DB.execute(SQL.User.set_user_token(db_user_id, 'NULL'))
    DB.commit()

    # 处理返回
    result.update({G_KEYWORD.Api.TransactionFieldKey.Message: '登出成功 !'})
    return ServerRequestHandler.successful(result)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 项目管理器 ]

"""
import shutil
from pathlib import Path

from ..const import G_CONST
from ..broadcast.awaken_log import G_LOG
from ..error.project import AwakenProjectInitNotExistsError
from ..error.project import AwakenEngineeringInitNotExistsError
from ...kit.common import TASK_TEMPLATE_MAP
from ...baseic.decorator.baseic import deco_singleton


SCRIPT_FILE_LOCAT_RULES = '*.awaken-*'
""" 脚本文件定位正则规则 """


# --------------------------------------------------------------------------
@deco_singleton
class ProjectFileHandle:
    """
    [ 项目管理器 ]

    ---
    描述
    - 用以管理本地的项目文件。
    
    """


    # ----------------------------------------------------------------------
    def get_projects(self) -> list[str]:
        """
        [ 获取本地项目列表 ]

        ---
        描述
        - 在本地工程目录中获取项目列表。

        ---
        返回
        - list[str] : 本地项目列表。

        """
        self._check_engineering_catalog()
        project_list = []
        for path in list(G_CONST.Path.ACTUALLY_CWD.glob('*')):
            try:
                if self._check_project_catalog(path):
                    project_list.append(path.stem)
            except AwakenProjectInitNotExistsError:
                ...
        return project_list


    # ----------------------------------------------------------------------
    def get_tasks(self, project_name: str) -> list[str]:
        """
        [ 获取本地项目的任务列表 ]

        ---
        描述
        - 在本地工程的项目目录中获取任务列表。

        ---
        返回
        - list[str] : 本地任务路径对象列表。

        """
        project_path = self._name_converted_to_path_object(project_name)
        self._check_project_catalog(project_path)
        return [name.stem for name in list(project_path.glob(SCRIPT_FILE_LOCAT_RULES))]


    # ----------------------------------------------------------------------
    def project_create(self, name: str | list) -> dict[str:bool]:
        """
        [ 项目创建 ]

        ---
        描述
        - 在本地工程目录中创建项目。

        ---
        参数
        - name : { str | list} - 项目名称, 传递列表则批量创建。

        ---
        返回
        - dict[str: bool] : 项目创建结果字典。
        
        """
        self._check_engineering_catalog()
        name = self._name_format_processing(name)
        create_results = {}
        for pn in name:
            try:
                pp = G_CONST.Path.ACTUALLY_CWD.joinpath(pn)
                if not pp.exists():
                    pp.mkdir(parents=True, exist_ok=True)
                    pp.joinpath(G_CONST.Name.FileName.ProjectInit).touch(mode=0o777, exist_ok=True)
                create_results.update({pn : True})

            except BaseException as error:
                G_LOG.error(f'创建项目异常 :: {error}')
                create_results.update({pn : False})

        return create_results


    # ----------------------------------------------------------------------
    def project_delete(self, name: str | list) -> dict[str:bool]:
        """
        [ 项目删除 ]

        ---
        描述
        - 在本地工程目录中创建项目。

        ---
        参数
        - name : { str | list} - 项目名称, 传递列表则批量删除。

        ---
        返回
        - dict[str: bool] : 项目删除结果字典。
        
        """
        self._check_engineering_catalog()
        name = self._name_format_processing(name)
        delete_results = {}
        for pn in name:
            try:
                pp = G_CONST.Path.ACTUALLY_CWD.joinpath(pn)
                if pp.exists():
                    shutil.rmtree(pp)
                    delete_results.update({pn : True})
                else:
                    raise

            except BaseException as error:
                G_LOG.error(f'删除项目异常 :: {error}')
                delete_results.update({pn : False})

        return delete_results


    # ----------------------------------------------------------------------
    def project_rename(self, old_name: str | list, new_name: str | list) -> dict[str:str]:
        """
        [ 项目重命名 ]

        ---
        描述
        - 在本地工程目录中更新项目的名称。

        ---
        参数
        - old_name : { str | list} - 旧项目名称, 传递列表则批量重命名, 如果传递列表时需与 new_name 的值一一对应。
        - new_name : { str | list} - 新项目名称, 传递列表则批量重命名, 如果传递列表时需与 old_name 的值一一对应。

        ---
        返回
        - dict[str: bool] : 项目重命名结果字典。
        
        """
        self._check_engineering_catalog()
        old_name = self._name_format_processing(old_name)
        new_name = self._name_format_processing(new_name)
        rename_results = {}
        for oldn, newn in dict(zip(old_name, new_name)).items():
            try:
                oldp = G_CONST.Path.ACTUALLY_CWD.joinpath(oldn)
                newp = G_CONST.Path.ACTUALLY_CWD.joinpath(newn)
                if oldp.exists() \
                and self._check_project_catalog(oldp) \
                and not newp.exists():
                    oldp.rename(newp)
                    rename_results.update({oldn : True})
                else:
                    raise

            except BaseException as error:
                G_LOG.error(f'项目重命名异常 :: {error}')
                rename_results.update({oldn : False})
        
        return rename_results


    # ----------------------------------------------------------------------
    def task_create(self, project_name: str, task_type: str, task_name: str | list) -> dict[str:str]:
        """
        [ 任务创建 ]

        ---
        描述
        - 在本地项目目录中创建任务文件。

        ---
        参数
        - project_name : { str } - 项目目录名称。
        - task_type : { str } - 任务类型。
        - task_name : { str | list} - 任务名称, 传递列表则批量创建。

        ---
        返回
        - dict[str: bool] : 任务创建结果字典。
        
        """
        task_name = self._name_format_processing(task_name)
        project_path = self._name_converted_to_path_object(project_name)
        self._check_project_catalog(project_path)

        if task_type.upper() not in TASK_TEMPLATE_MAP:
            raise

        create_results = {}
        for btn in task_name:
            try:
                task_name = f'{btn}.awaken-{task_type.lower()}'
                task_path = project_path.joinpath(task_name)
                if not task_path.exists():
                    task_template = TASK_TEMPLATE_MAP[task_type.upper()]
                    task_template = task_template.replace('#TASK_NAME#', btn)
                    task_template = task_template.replace('#TASK_TYPE#', task_type.upper())
                    task_path.touch(mode=0o777)
                    task_path.write_bytes(bytes(task_template, encoding='UTF8'))
                create_results.update({btn : True})

            except BaseException as error:
                G_LOG.error(f'创建任务异常 :: {error}')
                create_results.update({btn : False})

        return create_results


    # ----------------------------------------------------------------------
    def task_delete(self, project_name: str, task_name: str | list) -> dict[str:str]:
        """
        [ 任务删除 ]

        ---
        描述
        - 在本地项目目录中删除任务文件。

        ---
        参数
        - project_name : { str } - 项目目录名称。
        - task_name : { str | list} - 任务名称, 传递列表则批量删除。

        ---
        返回
        - dict[str: bool] : 任务删除结果字典。
        
        """
        task_name = self._name_format_processing(task_name)
        project_path = self._name_converted_to_path_object(project_name)
        self._check_project_catalog(project_path)

        delete_results = {}
        task_glob = list(project_path.glob(SCRIPT_FILE_LOCAT_RULES))
        project_tasks = dict(zip([n.stem for n in task_glob], task_glob))

        for task_name in task_name:
            try:
                if task_name in project_tasks.keys():
                    project_tasks[task_name].unlink()
                    delete_results.update({task_name : True})
                else:
                    raise

            except BaseException as error:
                G_LOG.error(f'删除任务异常 :: {error}')
                delete_results.update({task_name : False})

        return delete_results


    # ----------------------------------------------------------------------
    def task_rename(self, project_name: str, old_task_name: str | list, new_task_name: str | list) -> dict[str:str]:
        """
        [ 任务重命名 ]

        ---
        描述
        - 在本地工程的项目目录中更新任务的名称。

        ---
        参数
        - project_name : { str } - 项目目录名称。
        - old_task_name : { str | list} - 旧任务名称, 传递列表则批量重命名, 如果传递列表时需与 new_name 的值一一对应。
        - new_task_name : { str | list} - 新任务名称, 传递列表则批量重命名, 如果传递列表时需与 old_name 的值一一对应。

        ---
        返回
        - dict[str: bool] : 项目重命名结果字典。
        
        """
        old_task_name = self._name_format_processing(old_task_name)
        new_task_name = self._name_format_processing(new_task_name)
        project_path = self._name_converted_to_path_object(project_name)
        self._check_project_catalog(project_path)

        rename_results = {}
        for oldn, newn in dict(zip(old_task_name, new_task_name)).items():
            try:
                project_file_glob = list(project_path.glob(SCRIPT_FILE_LOCAT_RULES))
                project_task_list = dict(zip([n.stem for n in project_file_glob], project_file_glob))
                if oldn in project_task_list.keys():
                    oldp = project_task_list[oldn]
                    newp = oldp.with_stem(newn)
                    oldp.rename(newp)
                    rename_results.update({oldn : True})
                else:
                    raise

            except BaseException as error:
                G_LOG.error(f'任务重命名异常 :: {error}')
                rename_results.update({oldn : False})
        
        return rename_results

    
    # ----------------------------------------------------------------------
    def _check_engineering_catalog(self, path: Path = None) -> bool:
        """ 
        [ 检查工程目录 ]

        ---
        描述
        - 检查当前运行域是否是工程根目录。

        ---
        参数
        - path : { Path } - 工程目录路径对象, 为空则表示当前运行域。

        """
        engineering_path = path if path else G_CONST.Path.ACTUALLY_CWD
        if not engineering_path.joinpath(G_CONST.Name.FileName.EngineeringInit).exists():
            raise AwakenEngineeringInitNotExistsError([
                '当前目录非工程目录 !',
                '通过 "awaken -init" 命令初始化工程 .'
            ])
        return True


    # ----------------------------------------------------------------------
    def _check_project_catalog(self, path: Path = None) -> bool:
        """ 
        [ 检查项目目录 ]

        ---
        描述
        - 检查当前运行域是否是项目根目录。

        ---
        参数
        - path : { Path } - 项目目录路径对象, 为空则表示当前运行域。

        """
        project_path: Path
        if path:
            if not path.exists():
                raise AwakenProjectInitNotExistsError([
                f'项目路径"{path}"不存在 !',
                '请于工程根目录中创建项目目录, 再创建任务 .'
            ])
            project_path = path
        else:
            project_path = G_CONST.Path.ACTUALLY_CWD

        if not project_path.joinpath(G_CONST.Name.FileName.ProjectInit).exists():
            raise AwakenProjectInitNotExistsError([
                '当前目录非工程的项目目录(缺少项目ini) !',
                '请于工程根目录中创建项目目录, 再进入项目目录中创建任务 .'
            ])
        return True
    

    # ----------------------------------------------------------------------
    def _name_converted_to_path_object(self, name: str = None):
        if not name:
            return G_CONST.Path.ACTUALLY_CWD
        else:
            return G_CONST.Path.ACTUALLY_CWD.joinpath(name)


    # ----------------------------------------------------------------------
    def _name_format_processing(self, name: str | list) -> list:
        if isinstance(name, str):
            name = [name]
        return name


# --------------------------------------------------------------------------
G_PROJECT_FILE_HANDLE = ProjectFileHandle()
""" 项目管理器全局实例 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
接口关键字
    
"""


# --------------------------------------------------------------------------
class UserFieldKey:
    """
    [ 用户字段名关键字 ]

    """
    UserId = 'user_id'
    """ 用户ID """

    UserNumber = 'user_number'
    """ 用户账号 """

    Password = 'password'
    """ 用户密码 """

    UserName = 'user_name'
    """ 用户名称 """

    Portrait = 'portrait'
    """ 用户头像 """

    Token = 'token'
    """ 用户Token """

    Role = 'role'
    """ 用户角色 """


# --------------------------------------------------------------------------
class ProjectFieldKey:
    """ 
    [ 项目字段名关键字 ]

    """
    Count = 'count'
    """ 项目数量 """

    List = 'list'
    """ 项目列表 """

    Pid = 'pid'
    """ 项目ID """

    Name = 'name'
    """ 项目名称 """

    Type = 'type'
    """ 项目类型 """

    Docs = 'docs'
    """ 项目说明 """

    State = 'state'
    """ 状态 """
    
    CreatedDate = 'created_date'
    """ 创建日期 """


# --------------------------------------------------------------------------
class TasksFieldKey:
    """
    [ 任务字段名关键字 ]

    """
    Tid = 'tid'
    """ 任务ID """

    Pid = 'pid'
    """ 项目ID """

    Name = 'name'
    """ 名称 """

    Docs = 'docs'
    """ 说明 """

    Type = 'type'
    """ 类型 """

    CreatedDate = 'created_date'
    """ 创建日期 """

    State = 'state'
    """ 状态 """

    ScriptJson = 'script_json'
    """ 脚本Json """

    RunNumber = 'run_number'
    """ 运行次数 """

    LastRunDate = 'last_run_date'
    """ 最后运行日期 """

    LastRunTime = 'last_run_time'
    """ 最后运行时间 """

    SpendTime = 'spend_time'
    """ 消耗时间 """
    

# --------------------------------------------------------------------------
class TaskRecordFieldKey:
    """
    [ 任务记录字段名关键字 ]

    """
    TRid = 'trid'
    """ 编号 """

    Tid = 'tid'
    """ 所属任务ID """

    Name = 'name'
    """ 名称 """

    Docs = 'docs'
    """ 说明 """

    Type = 'type'
    """ 类型 """

    CreatedDate = 'created_date'
    """ 创建日期 """

    State = 'state'
    """ 状态 """

    StartTime = 'start_time'
    """ 开始时间 """

    EndTime = 'end_time'
    """ 结束时间 """

    SpendTime = 'spend_time'
    """ 消耗时间 """

    Process = 'process'
    """ 过程 """


# --------------------------------------------------------------------------
class CaseRecordFieldKey:
    """
    [ 用例记录字段名关键字 ]

    """
    CRid = 'crid'
    """ 编号 """

    TRid = 'trid'
    """ 所属任务记录ID """

    Name = 'name'
    """ 名称 """

    Docs = 'docs'
    """ 说明 """
    
    Type = 'type'
    """ 类型 """

    CreatedDate = 'created_date'
    """ 创建日期 """

    State = 'state'
    """ 状态 """

    StartTime = 'start_time'
    """ 开始时间 """

    EndTime = 'end_time'
    """ 结束时间 """

    SpendTime = 'spend_time'
    """ 消耗时间 """

    Process = 'process'
    """ 过程 """


# --------------------------------------------------------------------------
class InterpreterFieldKey:
    """
    [ 解释器字段名关键字 ]

    """
    TaskType = 'task_type'
    """ 任务类型 """
    
    Instructions = 'instructions'
    """ 指令集 """


# --------------------------------------------------------------------------
class ProcessingFieldKey:
    """
    [ 数据处理字段名关键字 ]

    """
    Url = 'url'
    """ URL """

    Document = 'document'
    """ DOM文档 """


# --------------------------------------------------------------------------
class TransactionFieldKey:
    """
    [ 事务处理字段名关键字 ]

    """
    Result = 'result'
    """ 结果 """

    Code = 'code'
    """ 状态编码 """

    Items = 'items'
    """ 项目 """

    Totai = 'total'
    """ 合计 """

    Data = 'data'
    """ 数据 """

    Count = 'count'
    """ 数量 """

    Message = 'message'
    """ 消息 """

    Time = 'time'
    """ 时间 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
类型相关异常

"""
from .baseic import AwakenBaseError


# --------------------------------------------------------------------------
class AwakenTypeDetectionFailedError(AwakenBaseError):
    """ [ 类型检测异常 ] """

    def __init__(
        self, 
        expect: str, 
        result: str,
    ):
        self.expect = expect
        self.result = result
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'期望类型:: {self.expect}, 意外得到:: {self.result} !'
        ]
        return super().__str__()


# --------------------------------------------------------------------------
class AwakenBrowserDriverNotSupportError(AwakenBaseError):
    """ [ 浏览器驱动不支持 ] """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 项目相关接口 ]

"""
import time

from flask import Response
from flask import Blueprint

from ..server_request_handler import URL_PREFIX
from ..server_request_handler import ServerRequestHandler
from ...db import DB
from ...db import SQL
from ....baseic.const import G_CONST
from ....baseic.keyword import G_KEYWORD
from ....baseic.decorator.web_api import deco_webapi_error_capture


blueprint_project = Blueprint('project', __name__)
""" 项目接口蓝图 """


POST_PROJECT_GET_PROJECT_COUNT = '/project/count'
""" 接口URL - 获取项目数量 """
POST_PROJECT_GET_PROJECT_LIST  = '/project/list'
""" 接口URL - 获取项目列表 """
POST_PROJECT_CREATION          = '/project/creation'
""" 接口URL - 项目创建 """
POST_PROJECT_MODIFY            = '/project/modify'
""" 接口URL - 项目编辑 """
POST_PROJECT_DELETE            = '/project/delete'
""" 接口URL - 项目删除 """


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_project.route(''.join([URL_PREFIX, POST_PROJECT_GET_PROJECT_COUNT]), methods=['POST'])
def post_count() -> Response:
    """
    [ POST - 查询项目数量 ]

    ---
    可选参数
    - pid          : { str } - 项目编号。
    - type         : { str } - 项目类型。
    - state        : { str } - 项目状态。
    - created_date : { str } - 项目创建日期, 例如 2022-02-02。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        optional_keys=[
            G_KEYWORD.Api.ProjectFieldKey.Pid,
            G_KEYWORD.Api.ProjectFieldKey.Type,
            G_KEYWORD.Api.ProjectFieldKey.State,
            G_KEYWORD.Api.ProjectFieldKey.CreatedDate,
        ]
    )

    # 从数据库中获取数据
    db_data = DB.execute(SQL.Project.get_count(paras), return_data_type='LIST')
    count = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Items)[0][0]

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Count: count
    })
    return ServerRequestHandler.successful(result)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_project.route(''.join([URL_PREFIX, POST_PROJECT_GET_PROJECT_LIST]), methods=['POST'])
def post_list() -> Response:
    """
    [ POST - 查询项目列表 ]

    ---
    可选参数
    - pid          : { str } - 项目编号。
    - type         : { str } - 项目类型。
    - state        : { str } - 项目状态。
    - created_date : { str } - 项目创建日期, 例如 2022-02-02。
    - page         : { str } - 当前页数。
    - number       : { str } - 单页数据量。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        optional_keys=[
            G_KEYWORD.Api.ProjectFieldKey.Pid,
            G_KEYWORD.Api.ProjectFieldKey.Type,
            G_KEYWORD.Api.ProjectFieldKey.State,
            G_KEYWORD.Api.ProjectFieldKey.CreatedDate,
            G_KEYWORD.Common.PagingDataFieldKey.Page,
            G_KEYWORD.Common.PagingDataFieldKey.Number,
        ]
    )

    # 从数据库中获取数据
    db_data = DB.execute(SQL.Project.get_list(paras))
    items = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Items)
    totai = db_data.get(G_KEYWORD.Common.TransactionFieldKey.Totai)

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Items: items,
        G_KEYWORD.Api.TransactionFieldKey.Totai: totai,
    })
    return ServerRequestHandler.successful(result)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_project.route(''.join([URL_PREFIX, POST_PROJECT_CREATION]), methods=['POST'])
def post_creation() -> Response:
    """
    [ POST - 项目创建 ]

    ---
    必要参数
    - type : { str } - 项目类型。
    - name : { str } - 项目名称。
    - docs : { str } - 项目说明。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        must_keys=[
            G_KEYWORD.Api.ProjectFieldKey.Type, 
            G_KEYWORD.Api.ProjectFieldKey.Name, 
            G_KEYWORD.Api.ProjectFieldKey.Docs
        ]
    )

    # 从数据库中写入数据
    pid = DB.execute(SQL.Project.creation(
        type=paras[G_KEYWORD.Api.ProjectFieldKey.Type],
        name=paras[G_KEYWORD.Api.ProjectFieldKey.Name],
        docs=paras[G_KEYWORD.Api.ProjectFieldKey.Docs],
        state=G_CONST.State.Project.Null,
        created_date=time.strftime('%Y-%m-%d')
    ))
    DB.commit()

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Message: '项目创建成功 !',
        G_KEYWORD.Api.ProjectFieldKey.Pid: pid
    })
    return ServerRequestHandler.successful(result)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_project.route(''.join([URL_PREFIX, POST_PROJECT_MODIFY]), methods=['POST'])
def post_modify() -> Response:
    """
    [ POST - 项目编辑 ]
 
    ---
    必要参数
    - pid  : { str } - 项目编号。
    - name : { str } - 项目名称。
    - docs : { str } - 项目说明。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        must_keys=[
            G_KEYWORD.Api.ProjectFieldKey.Pid,
            G_KEYWORD.Api.ProjectFieldKey.Name,
            G_KEYWORD.Api.ProjectFieldKey.Docs,
        ]
    )
    
    # 从数据库中写入数据
    DB.execute(SQL.Project.modify(
        pid=paras.get(G_KEYWORD.Api.ProjectFieldKey.Pid),
        name=paras.get(G_KEYWORD.Api.ProjectFieldKey.Name),
        docs=paras.get(G_KEYWORD.Api.ProjectFieldKey.Docs),
    ))
    DB.commit()

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Message: '项目编辑成功 !'
    })
    return ServerRequestHandler.successful(result)


# --------------------------------------------------------------------------
@deco_webapi_error_capture
@blueprint_project.route(''.join([URL_PREFIX, POST_PROJECT_DELETE]), methods=['POST'])
def post_delete() -> Response:
    """
    [ POST - 项目删除 ]

    ---
    必要参数
    - pid  : { str } - 项目编号。

    """
    result = {}

    # 解析请求
    paras = ServerRequestHandler.analysis_request_parameter(
        must_keys=[
            G_KEYWORD.Api.ProjectFieldKey.Pid,
        ]
    )
    pid = paras.get(G_KEYWORD.Api.ProjectFieldKey.Pid)

    # 从数据库中写入数据
    DB.execute(SQL.Project.delete(pid=pid))
    DB.execute(SQL.Tasks.delete_form_project(pid=pid))
    DB.commit()

    # 处理返回
    result.update({
        G_KEYWORD.Api.TransactionFieldKey.Message: '项目删除成功 !'
    })
    return ServerRequestHandler.successful(result)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
[ Awaken 自动化框架 ]

---
描述
    - Awaken 是一个功能自动化测试框架, 将支持 WEB、API、APP 等平台。
    - 关键字驱动技术是该框架的一个重要思想, 它将测试用例设计与自动化实现开发分离开来;
    - 测试人员不需要具备很深的脚本语言知识, 应用该框架自实现的一套精简的关键字语法即可快速编写用例逻辑。

"""


# --------------------------------------------------------------------------
class AwakenDetails:
    """ 
    [ Awaken详情结构体 ] 
    
    """
    Name     = 'awaken2'
    """ 项目名称 """
    Version  = '0.2.4'
    """ 项目版本 """
    Desc     = 'Awaken 是一个功能自动化测试框架, 将支持 WEB、API、APP 等平台。'
    """ 项目简介 """
    Url      = 'https://gitee.com/catcat7/awaken2'
    """ 项目URL """
    Author   = 'chenjiancheng'
    """ 项目作者 """
    Email    = 'quinn.7@foxmail.com'
    """ 作者邮箱 """
    Keywords = ['awaken2', 'python', 'testing', 'automation', 'testautomation']
    """ 项目关键字 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
[ 环境相关通用函数 ]

"""
from ..const import G_CONST
from ..broadcast.awaken_log import G_LOG
from ..environment.web.browser_driver_handle import G_BROWSER_DRIVER_HANDLE


# --------------------------------------------------------------------------
def environment_check_browser_driver_exists():
    """
    [ 环境检查 - 安装浏览器驱动 ]

    ---
    描述
    - 检查本地是否下载了WEB引擎所需的浏览器驱动, 否则下载。

    """
    # 检查本地已安装的浏览器驱动
    complete   = []
    uncomplete = []
    local_driver = G_BROWSER_DRIVER_HANDLE.inspect()

    for dk, dv in local_driver.items():
        if dv:
            complete.append(dk)
        else:
            uncomplete.append(dk)

    # 安装未安装的浏览器驱动
    if len(uncomplete) > 0:
        G_BROWSER_DRIVER_HANDLE.install(uncomplete)


# --------------------------------------------------------------------------
def environment_check_dependent_files_exists():
    """
    [ 环境检查 - 创建工程依赖 ]

    """
    # 创建目录
    for path in [
        G_CONST.Path.DirPath.Data,
        G_CONST.Path.DirPath.Logs,
        G_CONST.Path.DirPath.BaseCode,
        G_CONST.Path.DirPath.Projects,
    ]:
        G_LOG.debug(f'创建工程依赖目录:: {str(path.resolve())}')
        path.mkdir(parents=True, exist_ok=True)

    # 创建文件
    for path in [
        G_CONST.Path.FilePath.EngineeringInit,
        G_CONST.Path.FilePath.EngineeringConfig,
        G_CONST.Path.FilePath.EngineeringDatabase,
    ]:
        G_LOG.debug(f'创建工程依赖文件:: {str(path.resolve())}')
        path.touch(mode=0o777, exist_ok=True)



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
路径常量

"""
from pathlib import Path

from . import const_name


ACTUALLY_CWD = Path().cwd()
""" Path对象 - 实际运行时路径 """

CONSOLE_CWD = ACTUALLY_CWD
""" Path对象 - 命令行运行时路径 """


# --------------------------------------------------------------------------
class DirPath:
    """ 
    [ 目录路径对象常量 ]

    """
    Data     = ACTUALLY_CWD.joinpath(const_name.DirName.EngineeringData)
    """ 数据目录 """

    Projects = Data.joinpath(const_name.DirName.EngineeringProjects)
    """ 项目目录 """

    Logs     = Data.joinpath(const_name.DirName.EngineeringLogs)
    """ 日志目录 """

    BaseCode = Data.joinpath(const_name.DirName.EngineeringBaseCode)
    """ 底层编码目录 """


# --------------------------------------------------------------------------
class FilePath:
    """
    [ 文件路径对象常量 ]
        
    """
    EngineeringInit     = ACTUALLY_CWD.joinpath(const_name.FileName.EngineeringInit)
    """ 工程标识文件 """

    EngineeringConfig   = DirPath.Data.joinpath(const_name.FileName.EngineeringConfig)
    """ 工程配置文件 """

    EngineeringDatabase = DirPath.Data.joinpath(const_name.FileName.EngineeringDatabase)
    """ 工程数据库文件 """

    ProjectInit         = ACTUALLY_CWD.joinpath(const_name.FileName.ProjectInit)
    """ 项目标识文件 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 配置器 ]

"""
import os
import yaml

from pathlib import Path

from ..const import G_CONST
from ..keyword import G_KEYWORD
from ..error.file import AwakenDependentFileIsMissing


# --------------------------------------------------------------------------
class Configure:
    """
    [ 配置器 ]
    
    """
    _config_path: Path
    """ 配置路径 """
    _config_data: dict
    """ 配置数据 """
    _default_dict: dict
    """ 缺省数据 """
    _protection_keys: list
    """ 受保护密匙列表 """


    # ----------------------------------------------------------------------
    def __init__(
        self, 
        config_path: Path, 
        default_dict: dict = None, 
        protection_keys: list = []
    ) -> ...:
        """
        [ 配置器 ]

        ---
        参数
        - config_path     : { Path } - 配置文件路径对象。
        - default_dict    : { dict } - 缺省数据字典。
        - protection_keys : { list } - 受保护密匙列表, 该列表中的 key 无法从配置器中删除。
        
        """
        self._config_path = config_path
        self._default_dict = default_dict
        self._protection_keys = protection_keys

        if not self._config_path.exists():
            raise AwakenDependentFileIsMissing(f'{ config_path }')

        self._read_config_file()


    # ----------------------------------------------------------------------
    def _read_config_file(self) -> ...:
        """
        [ 读取配置文件 ]
        
        """
        # 配置文件如果为空则创建写入缺省配置数据
        if os.path.getsize(self._config_path) == 0:
            self._write_default_data()

        with open(file=self._config_path, mode='r', encoding='UTF-8') as file:
            config_data = yaml.load(file, Loader=yaml.FullLoader)
            self._config_data = config_data if config_data else {}


    # ----------------------------------------------------------------------
    def _write_default_data(self) -> ...:
        """
        [ 写入缺省数据 ]
        
        """
        if self._default_dict:
            self._config_data = self._default_dict
            self.dump()


    # ----------------------------------------------------------------------
    def dump(self) -> ...:
        """
        [ 存储配置 ]

        """
        with open(file=self._config_path, mode='w', encoding='UTF-8') as file:
            yaml.dump(self._config_data, file, allow_unicode=True)


    # ----------------------------------------------------------------------
    def data(self) -> dict:
        return self._config_data


    # ----------------------------------------------------------------------
    def keys(self) -> list:
        return self._config_data.keys()

    
    # ----------------------------------------------------------------------
    def get(self, key: str) -> str | None:
        """
        [ 获取配置项 ]

        ---
        参数
        - key : { str } - 配置项Key。

        """
        if key in self._config_data.keys():
            return self._config_data[key]
        else:
            return None


    # ----------------------------------------------------------------------
    def set(self, key: str, value: str) -> ...:
        """
        [ 设置配置项 ]

        """
        self._config_data.update({key : value})

    
    # ----------------------------------------------------------------------
    def delete(self, key: str) -> ...:
        """
        [ 删除配置项 ]

        """
        if key in self._protection_keys:
            ...
        else:
            del self._config_data[key]


# --------------------------------------------------------------------------
_CONFIG_DEFAULT_DICT = {
    G_KEYWORD.Common.Config.Debug               : True,  # 是否开启调试
    G_KEYWORD.Common.Config.EngineQueueMaxCount : 3,     # 引擎队列最大数量
    G_KEYWORD.Common.Config.TaskQueueMaxCount   : 50,    # 任务队列最大数量
    G_KEYWORD.Common.Config.CpuPropertyTime     : 1,     # CPU资源检索时隔
    G_KEYWORD.Common.Config.CpuPropertyCeiling  : 30,    # CPU资源检索阈值
}
""" 配置器缺省字典 """

G_CONFIGURE: Configure = Configure(
    config_path=G_CONST.Path.FilePath.EngineeringConfig,
    default_dict=_CONFIG_DEFAULT_DICT,
    protection_keys=[key for key in _CONFIG_DEFAULT_DICT.keys()]
)
""" Awaken配置器全局实例 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 模板渲染器 ]

"""


# --------------------------------------------------------------------------
class TemplateStyle:
    """ 
    [ 模板样式对象 ]

    """
    tab_horizontal   = '━'
    """ 制表符::水平 """
    tab_side         = '┃'
    """ 制表符::边柱 """
    tab_side_left    = '┣'
    """ 制表符::左柱 """
    tab_side_right   = '┫'
    """ 制表符::右柱 """
    tab_top_left     = '┏'
    """ 制表符::左上 """
    tab_bottom_left  = '┗'
    """ 制表符::左下 """
    tab_top_right    = '┓'
    """ 制表符::右上 """
    tab_bottom_right = '┛'
    """ 制表符::右下 """
    size = 70
    """ 表格长度 """


# --------------------------------------------------------------------------
class TemplateRendering:
    """
    [ 模板渲染器 ]

    ---
    描述
    - 采用模板的方式美化命令行信息输出。

    """
    _tab_style: TemplateStyle
    """ 模板样式 """


    # ----------------------------------------------------------------------
    def __init__(self, style: TemplateStyle = None) -> ...:
        self._tab_style = style if style != None else TemplateStyle
        self._tabulation_top     = ''.join([self._tab_style.tab_top_left, self._tab_style.tab_horizontal*self._tab_style.size, self._tab_style.tab_top_right])
        self._tabulation_bearing = ''.join([self._tab_style.tab_side_left, self._tab_style.tab_horizontal*self._tab_style.size, self._tab_style.tab_side_right])
        self._tabulation_bottom  = ''.join([self._tab_style.tab_bottom_left, self._tab_style.tab_horizontal*self._tab_style.size, self._tab_style.tab_bottom_right])


    # ----------------------------------------------------------------------
    def render_print(
        self, 
        source: list,
        title: str = None,
        is_show_number: bool = True,
        is_keep_space_top_and_bottom: bool = True,
        ) -> ...:
        """
        [ 渲染打印 ]

        ---
        描述
        - 打印模板表格。

        ---
        参数
        - source                       : { list } - 字符串列表。
        - title                        : { str }  - 模板抬头, 默认为空。
        - is_show_number               : { bool } - 是否显示编号, 默认为 True。
        - is_keep_space_top_and_bottom : { bool } - 是否保持上下间距, 默认为 True。
        
        """
        cmd_template = self.draw(source, title, is_show_number, is_keep_space_top_and_bottom)
        print('\n')
        for string in cmd_template:
            print(string)


    # ----------------------------------------------------------------------
    def draw(
        self, 
        source: list,
        title: str = None,
        is_show_number: bool = True,
        is_keep_space_top_and_bottom: bool = True,
        ) -> list:
        """
        [ 描绘 ]
        
        ---
        描述
        - 绘制模板表格。
        
        ---
        参数
        - source                       : { list } - 字符串列表。
        - title                        : { str }  - 模板抬头, 默认为空。
        - is_show_number               : { bool } - 是否显示编号, 默认为 True。
        - is_keep_space_top_and_bottom : { bool } - 是否保持上下间距, 默认为 True。
        
        ---
        返回
        - list : 模板字符串列表。
        
        """
        # 绘制模板抬头
        final_output_list = [self._tabulation_top]

        if title:
            if self._calculate_character_exceed(title):
                title, _ = self._clipping_out_of_length_strings(title)
            final_output_list.append(self._process_template_content(title))
            final_output_list.append(self._tabulation_bearing)

        if is_keep_space_top_and_bottom:
            source = [' ', *source, ' ']

        # 处理最终输出列表
        new_number = 0
        for str in source:
            if is_show_number and str != ' ':
                new_number += 1
                str = f'{ new_number } : { str }'
            while 1:
                if self._calculate_character_exceed(str):
                    str1, str = self._clipping_out_of_length_strings(str)
                    final_output_list.append(self._process_template_content(str1))
                    continue
                final_output_list.append(self._process_template_content(str))
                break
        final_output_list.append(self._tabulation_bottom)
        return final_output_list


    # ----------------------------------------------------------------------
    def _process_template_content(self, string: str) -> str:
        """
        [ 处理模板内容 ]

        ---
        描述
        - 根据Unicode编码判断字符是否中文以计算偏移量, 加以处理模板内容。

        ---
        返回
        - str : 处理完毕的模板内容。

        """
        offset = self._calculate_character_offset(string)
        return ''.join([
            self._tab_style.tab_side, ' ',
            string,
            ' '*((self._tab_style.size - len(string) - 2) - offset),
            ' ', self._tab_style.tab_side,
        ])


    # ----------------------------------------------------------------------
    def _calculate_character_offset(self, cha: str) -> int:
        """
        [ 计算字符偏移量 ]
        
        ---
        描述
        - 根据Unicode编码表判断字符是否中文以计算偏移量。

        ---
        返回
        - int : 字符偏移量。

        """
        offset = 0
        for s in cha:
            if u'\u4e00' <= s <= u'\u9fff':
                offset += 1
        return offset


    # ----------------------------------------------------------------------
    def _calculate_character_exceed(self, cha: str) -> bool:
        """
        [ 计算字符是否超出 ]

        ---
        描述
        - 根据模板表格的长度计算字符是否超出。

        ---
        返回
        - bool : 判断布尔值。

        """
        cha_number = len(cha)
        cha_offset = self._calculate_character_offset(cha)
        calculated_value = (self._tab_style.size - 2) - (cha_number + cha_offset)

        if calculated_value < 0:
            return True
        else:
            return False


    # ----------------------------------------------------------------------
    def _clipping_out_of_length_strings(self, string: str, accuracy: int = 0.01) -> tuple[str, str]:
        """
        [ 剪裁超出长度的字符串 ]

        """
        try:
            if accuracy >= 1 or accuracy < 0.01:
                raise Exception('参数 accuracy 只允许传入 0.01 - 0.99 的值.')

            first_decimal_point = int(str(accuracy)[2])
            truncation = 0.01 if first_decimal_point == 0 else 0.1
            while 1:
                subscript = int(len(string)*(1-truncation))
                if self._calculate_character_exceed(string[0: subscript]):
                    truncation += accuracy
                    continue
                str1 = string[0: subscript]
                str2 = string[subscript:]
                return str1, str2

        except BaseException as error:
            print(str(error))
            return self._clipping_out_of_length_strings(string, 1)


# --------------------------------------------------------------------------
G_TEMPLATE_RENDERING: TemplateRendering = TemplateRendering()
""" 模板渲染器全局实例 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
状态常量

"""


# --------------------------------------------------------------------------
class Project:
    """
    [ 项目状态常量 ]

    """
    Null = '未知'
    """ 未知 """


# --------------------------------------------------------------------------
class Result:
    """
    [ 结果常量 ]

    """
    Null      = '未知'
    """ 未知 """

    Success   = '成功'
    """ 成功 """

    Unsuccess = '失败'
    """ 失败 """

    Error     = '异常'
    """ 异常 """


# --------------------------------------------------------------------------
class Switch:
    """
    [ 开关常量 ]

    """
    Null = '未知'
    """ 未知 """

    Off  = '关'
    """ 关 """

    On   = '开'
    """ 开 """


# --------------------------------------------------------------------------
class RunProcess:
    """
    [ 运行过程常量 ]

    """
    Null  = '未知'
    """ 未知 """
    
    Start = '开始'
    """ 开始 """

    End   = '结束'
    """ 结束 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ 日志模块 ]

"""
import time
import logging

from ..const import G_CONST
from ..decorator.baseic import deco_singleton
from ..error.broadcast import AwakenLogRelevanError


_LOG_BASEIC_LEVEL       = 'INFO'
""" 日志默认流 Level 等级 """
_LOG_FILE_HANDLE_LEVELS = ['debug', 'info', 'warning', 'error', 'critical']
""" 日志文件流 Level 等级组 """
_LOG_OUTPUT_STRUCTURE   = logging.Formatter('[%(asctime)s] [%(levelname)7s] %(message)s')
""" 日志输出格式 """


# --------------------------------------------------------------------------
@deco_singleton
class AwakenLog:
    """
    [ 日志模块 ]
    
    """
    _logger: logging.Logger
    """ 日志记录器 """


    # ----------------------------------------------------------------------
    def __init__(self) -> ...:
        """
        [ 日志模块 ]
        
        """
        try:
            # 创建日志记录器
            self._logger = logging.getLogger()

            # 设置日志记录器 Level 等级
            self._logger.setLevel(_LOG_BASEIC_LEVEL)
            self._logger.handlers.clear()

            # 以当天日期为名称创建日志存放目录
            current_date = time.strftime('%Y-%m-%d', time.localtime())
            current_date_dir_path = G_CONST.Path.DirPath.Logs.joinpath(current_date)
            if not current_date_dir_path.exists():
                current_date_dir_path.mkdir(parents=True, exist_ok=True)

            # 构建日志默认流
            baseic_handle = logging.StreamHandler()
            baseic_handle.setLevel(_LOG_BASEIC_LEVEL)
            baseic_handle.setFormatter(_LOG_OUTPUT_STRUCTURE)
            self._logger.addHandler(baseic_handle)

            # 构建日志文件流
            for level in _LOG_FILE_HANDLE_LEVELS:
                file_path = current_date_dir_path.joinpath(f'{ level }.log')
                file_handle = logging.FileHandler(file_path)
                file_handle.setLevel(level.upper())
                file_handle.setFormatter(_LOG_OUTPUT_STRUCTURE)
                self._logger.addHandler(file_handle)

        except Exception as error:
            raise AwakenLogRelevanError(str(error))


    # ----------------------------------------------------------------------
    def out_map(self, level: str, message: str | list) -> ...:
        """
        [ 日志映射输出 ]

        ---
        描述
        - 根据传递的 level 映射对应等级日志打印。

        ---
        参数
        - level : { str } - 日志 Level 等级。
        - message : { str | list } - 输出的日志信息。

        ---
        示例
        >>> Log = AwakenLog()
        >>> Log.out_map(level='error', message='message')

        """
        try:
            type(self).__dict__[level](self, message)

        except KeyError as error:
            error_message = [f'日志 Level 等级 { str(error) } 不存在, 仅支持以下 Level 等级 !']
            error_message.extend(_LOG_FILE_HANDLE_LEVELS)
            raise AwakenLogRelevanError(error_message)


    # ----------------------------------------------------------------------
    def debug(self, message: str | list) -> ...:
        """
        [ 输出 DEBUG 日志 ]

        ---
        参数
        - message : { str | list } - 输出的日志信息。

        """
        message = [message] if isinstance(message, str) else message
        for m in message:
            self._logger.debug(m)


    # ----------------------------------------------------------------------
    def info(self, message: str | list) -> ...:
        """
        [ 输出 INFO 日志 ]

        ---
        参数
        - message : { str | list } - 输出的日志信息。

        """
        message = [message] if isinstance(message, str) else message
        for m in message:
            self._logger.info(m)


    # ----------------------------------------------------------------------
    def warning(self, message: str | list) -> ...:
        """
        [ 输出 WARNING 日志 ]

        ---
        参数
        - message : { str | list } - 输出的日志信息。

        """
        message = [message] if isinstance(message, str) else message
        for m in message:
            self._logger.warning(m)


    # ----------------------------------------------------------------------
    def error(self, message: str | list) -> ...:
        """
        [ 输出 ERROR 日志 ]

        ---
        参数
        - message : { str | list } - 输出的日志信息。

        """
        message = [message] if isinstance(message, str) else message
        for m in message:
            self._logger.error(m)
        

    # ----------------------------------------------------------------------
    def critical(self, message: str | list) -> ...:
        """
        [ 输出 CRITICAL 日志 ]

        ---
        参数
        - message : { str | list } - 输出的日志信息。

        """
        message = [message] if isinstance(message, str) else message
        for m in message:
            self._logger.critical(m)

    
# --------------------------------------------------------------------------
G_LOG: AwakenLog = AwakenLog()
""" 日志模块全局实例 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
名称常量

"""


# --------------------------------------------------------------------------
class DirName:
    """
    [ 目录名称常量 ]
    
    """

    EngineeringData     = 'EngineeringData'
    """ 工程数据目录 """

    EngineeringLogs     = 'Logs'
    """ 工程日志目录 """

    EngineeringProjects = 'Projects'
    """ 项目目录 """

    EngineeringBaseCode = 'BaseCode'
    """ 工程底层编码目录 """


# --------------------------------------------------------------------------
class FileName:
    """
    [ 文件名称常量 ]
    
    """

    EngineeringInit     = 'AwakenEngineering.ini'
    """ 工程INI文件 """

    EngineeringConfig   = 'AwakenConfig.yaml'
    """ 工程配置文件 """

    EngineeringDatabase = 'AwakenDatabase.db'
    """ 工程数据库文件 """

    ProjectInit         = 'AwakenProject.ini'
    """ 项目INI文件 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
[ SQL蓝图 - 用例记录相关 ]

"""
from typing import Any

from ..sqlite_statement_handle import G_SQLITE_STATEMENT_HANDLE
from ....baseic.const import G_CONST
from ....baseic.keyword import G_KEYWORD


# --------------------------------------------------------------------------
class SqlCaseRecord(object):
    """
    [ SQL蓝图 - 用例记录相关 ]

    """


    # ----------------------------------------------------------------------
    @staticmethod
    def get_count(paras: dict = {}) -> Any:
        """
        [ 查询用例编号数量 ]

        ---
        可选参数
        - crid          : { str } - 用例记录编号。
        - trid          : { str } - 任务记录编号。
        - name          : { str } - 用例记录名称。
        - type          : { str } - 用例记录类型。
        - created_date  : { str } - 用例记录创建日期, 例如 2022-02-02。
        - state         : { str } - 用例记录状态。
        - start_time    : { str } - 用例记录开始时间。
        - end_time      : { str } - 用例记录结束时间。

        """
        selecter = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.CaseRecord)
        selecter.get(G_KEYWORD.DataBase.CaseRecordFieldKey.CRid, G_KEYWORD.Sqlite.SelectEvent.Count)
        selecter.filter_equal_batch(paras)
        return selecter


    # ----------------------------------------------------------------------
    @staticmethod
    def get_list(paras: dict = {}) -> Any:
        """
        [ 查询用例记录列表 ]

        ---
        可选参数
        - crid         : { str } - 用例记录编号。
        - trid         : { str } - 任务记录编号。
        - name         : { str } - 用例记录名称。
        - type         : { str } - 用例记录类型。
        - created_date : { str } - 用例记录创建日期, 例如 2022-02-02。
        - state        : { str } - 用例记录状态。
        - start_time   : { str } - 用例记录开始时间。
        - end_time     : { str } - 用例记录结束时间。
        - page         : { str } - 当前页数。
        - number       : { str } - 单页数据量。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.CaseRecord)
        sql_handler.filter_equal_batch(paras)
        sql_handler.set_sort_desc(G_KEYWORD.DataBase.CaseRecordFieldKey.CRid)
        page   = paras.get(G_KEYWORD.Common.PagingDataFieldKey.Page)
        number = paras.get(G_KEYWORD.Common.PagingDataFieldKey.Number)
        sql_handler.set_data_limit(page, number)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def get_spend_time(paras: dict = {}) -> Any:
        """
        [ 查询任务记录的消耗时间 ]

        ---
        可选参数
        - crid          : { str } - 用例记录编号。
        - trid          : { str } - 任务记录编号。
        - name          : { str } - 用例记录名称。
        - type          : { str } - 用例记录类型。
        - created_date  : { str } - 用例记录创建日期, 例如 2022-02-02。
        - result        : { str } - 用例记录结果。
        - start_time    : { str } - 用例记录开始时间。
        - end_time      : { str } - 用例记录结束时间。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.select(G_KEYWORD.DataBase.TableName.CaseRecord)
        sql_handler.get(G_KEYWORD.DataBase.CaseRecordFieldKey.SpendTime, G_KEYWORD.Sqlite.SelectEvent.Sum)
        sql_handler.filter_equal_batch(paras)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def creation(
        trid: int,
        name: str,
        docs: str,
        type: str,
        created_date: str,
        start_time: str,
        state: str = G_CONST.State.Result.Null
    ) -> Any:
        """
        [ 创建用例记录 ]

        ---
        参数
        - trid         : { str } - 任务记录编号。
        - name         : { str } - 用例记录名称。
        - docs         : { str } - 用例记录说明。
        - type         : { str } - 用例记录类型。
        - created_date : { str } - 用例记录创建日期, 例如 2022-02-02。
        - start_time   : { str } - 用例记录开始时间。
        - state        : { str } - 用例记录状态。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.insert(G_KEYWORD.DataBase.TableName.CaseRecord)
        sql_handler.deduction([
            'NULL', trid, name, docs, type, created_date, state, start_time, 
            'NULL', 'NULL', 'NULL'
        ])
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def update(
        crid: int,
        state: str,
        end_time: str,
        spend_time: int,
        process: str
    ) -> Any:
        """
        [ 更新用例记录 ]

        ---
        参数
        - crid       : { int } - 用例记录编号。
        - state      : { str } - 用例记录状态。
        - end_time   : { str } - 用例记录结束时间。
        - spend_time : { int } - 用例记录消耗时间。
        - process    : { str } - 用例记录过程。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.update(G_KEYWORD.DataBase.TableName.CaseRecord)
        sql_handler.push(G_KEYWORD.DataBase.CaseRecordFieldKey.State, state)
        sql_handler.push(G_KEYWORD.DataBase.CaseRecordFieldKey.EndTime, end_time)
        sql_handler.push(G_KEYWORD.DataBase.CaseRecordFieldKey.SpendTime, spend_time)
        sql_handler.push(G_KEYWORD.DataBase.CaseRecordFieldKey.Process, process)
        sql_handler.filter_equal(G_KEYWORD.DataBase.CaseRecordFieldKey.CRid, crid)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def delete(paras: dict = {}) -> Any:
        """
        [ 删除用例记录 ]
        
        ---
        可选参数
        - trid         : { str } - 任务记录编号。
        - name         : { str } - 用例记录名称。
        - type         : { str } - 用例记录类型。
        - created_date : { str } - 用例记录创建日期, 例如 2022-02-02。
        - start_time   : { str } - 用例记录开始时间。
        - state        : { str } - 用例记录状态。

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.delete(G_KEYWORD.DataBase.TableName.CaseRecord)
        sql_handler.filter_equal_batch(paras)
        return sql_handler


    # ----------------------------------------------------------------------
    @staticmethod
    def delete_all() -> Any:
        """
        [ 删除全部项目 ]

        """
        sql_handler = G_SQLITE_STATEMENT_HANDLE.delete(G_KEYWORD.DataBase.TableName.CaseRecord)
        return sql_handler



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" 
解释器模块相关异常

"""
from .baseic import AwakenBaseError


# --------------------------------------------------------------------------
class AwakenInterpreterParseException(AwakenBaseError):
    """ [ 解释器异常::解析出现异常 ] """


# --------------------------------------------------------------------------
class AwakenInterpreterFileIsEmpty(AwakenBaseError):
    """ [ 解释器异常::脚本文件为空 ] """


# --------------------------------------------------------------------------
class AwakenInterpreterSymbolWrongFul(AwakenBaseError):
    """ [ 解释器异常::语法符不合法 ] """

    def __init__(
        self, 
        codeline_number: int, 
        codeline_content: str,
        error_symbol: str,
    ):
        self.codeline_number = codeline_number
        self.codeline_content = codeline_content
        self.error_symbol = error_symbol
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'编码行数: {self.codeline_number}',
            f'编码语句: {self.codeline_content}',
            f'上述语句中的语法符 "{ self.error_symbol }" 不合法 .',
            '通过 awaken -help 或相关的文档寻求帮助 .'
        ]
        return super().__str__()


# --------------------------------------------------------------------------
class AwakenInterpreterStatementWrongFul(AwakenBaseError):
    """ [ 解释器异常::关键字不合法 ] """

    def __init__(
        self, 
        codeline_number: int, 
        codeline_content: str,
        error_keyword: str,
    ):
        self.codeline_number = codeline_number
        self.codeline_content = codeline_content
        self.error_keyword = error_keyword
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'编码行数: {self.codeline_number}',
            f'编码语句: {self.codeline_content}',
            f'上述语句中的关键字 "{ self.error_keyword }" 不合法 .',
            '通过 awaken -help 或相关的文档寻求帮助 .'
        ]
        return super().__str__()


# --------------------------------------------------------------------------
class AwakenInterpreterStatementNoParameters(AwakenBaseError):
    """ [ 解释器异常::语法声明关键字无参数 ] """

    def __init__(
        self, 
        codeline_number: int, 
        codeline_content: str,
        error_keyword: str,
    ):
        self.codeline_number = codeline_number
        self.codeline_content = codeline_content
        self.error_keyword = error_keyword
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'编码行数: {self.codeline_number}',
            f'编码语句: {self.codeline_content}',
            f'上述语句中的关键字 "{ self.error_keyword }" 没有携带参数 .',
            '通过 awaken -help 或相关的文档寻求帮助 .'
        ]
        return super().__str__()


# --------------------------------------------------------------------------
class AwakenInterpreterCaseNotLogicAssert(AwakenBaseError):
    """ [ 解释器异常::用例未断言 ] """

    def __init__(self, case_name: str):
        self.case_name = case_name
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'用例 "{ self.case_name }" 未完成断言 .',
            '通过 awaken -help 或相关的文档寻求帮助 .'
        ]
        return super().__str__()


# --------------------------------------------------------------------------
class AwakenInterpreterBaseCodeLineTypeError(AwakenBaseError):
    """ [ 解释器异常::语法底层编码行类型异常 ] """

    def __init__(self, base_code_line_type: str):
        self.base_code_line_type = base_code_line_type
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'无法识别的编码行类型 "{ self.base_code_line_type }" .',
        ]
        return super().__str__()


# --------------------------------------------------------------------------
class AwakenInterpreterMethodWrongFul(AwakenBaseError):
    """ [ 解释器异常::方法不合法 ] """

    def __init__(
        self, 
        codeline_number: int, 
        codeline_content: str,
        error_method_name: str,
    ):
        self.codeline_number = codeline_number
        self.codeline_content = codeline_content
        self.error_method_name = error_method_name
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'编码行数: {self.codeline_number}',
            f'编码语句: {self.codeline_content}',
            f'上述语句中的方法关键字 "{ self.error_method_name }" 不是合法的方法 .',
            '通过 awaken -help 或相关的文档寻求帮助 .'
        ]
        return super().__str__()


# --------------------------------------------------------------------------
class AwakenInterpreterNamespaceNodeNotExist(AwakenBaseError):
    """ [ 解释器异常::语法命名空间不存在节点 ] """

    def __init__(self, node_name: str):
        self.node_name = node_name
        super().__init__()

    def __str__(self) -> str:
        self.bas_message = [
            f'命名空间不存在节点 "{ self.node_name }" .',
        ]
        return super().__str__()



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
通用字段关键字
    
"""


# --------------------------------------------------------------------------
class Config:
    """
    [ 配置关键字 ]
        
    """
    Debug = 'debug'
    """ 调试模式 """

    IsWriteDatabase = 'is_write_database'
    """ 是否写入数据库 """

    EngineQueueMaxCount = 'engine_queue_max_count'
    """ 引擎队列最大数量 """

    TaskQueueMaxCount = 'task_queue_max_count'
    """ 任务队列最大数量 """

    CpuPropertyTime = 'cpu_property_time'
    """ CPU资源检索时隔 """

    CpuPropertyCeiling = 'cpu_property_ceiling'
    """ CPU资源检索阈值 """


# --------------------------------------------------------------------------
class TransactionFieldKey:
    """
    [ 事务处理字段名关键字 ]

    """
    Result = 'result'
    """ 结果 """

    Items = 'items'
    """ 项目 """

    Data = 'data'
    """ 数据 """

    Totai = 'total'
    """ 合计 """

    Count = 'count'
    """ 数量 """


# --------------------------------------------------------------------------
class PagingDataFieldKey:
    """
    [ 分页数据字段名关键字 ]

    """
    Page = 'page'
    """ 页数 """

    Number = 'number'
    """ 数据量 """



# Copyright 2022 quinn.7@foxmail.com All rights reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
关键字索引

"""
from . import keyword_api
from . import keyword_common
from . import keyword_database
from . import keyword_script
from . import keyword_sqlite


# --------------------------------------------------------------------------
class AwakenKeyword:
    """
    [ 关键字索引 ]

    """


    # ----------------------------------------------------------------------
    @property
    def Api(self) -> keyword_api:
        """
        [ 接口关键字 ]

        """
        return keyword_api


    # ----------------------------------------------------------------------
    @property
    def Common(self) -> keyword_common:
        """
        [ 通用关键字 ]

        """
        return keyword_common


    # ----------------------------------------------------------------------
    @property
    def DataBase(self) -> keyword_database:
        """
        [ 数据库关键字 ]

        """
        return keyword_database


    # ----------------------------------------------------------------------
    @property
    def Script(self) -> keyword_script:
        """
        [ 脚本关键字 ]

        """
        return keyword_script


    # ----------------------------------------------------------------------
    @property
    def Sqlite(self) -> keyword_sqlite:
        """
        [ SQLite 语句关键字 ]

        """
        return keyword_sqlite


# --------------------------------------------------------------------------
G_KEYWORD: AwakenKeyword  = AwakenKeyword()
""" 关键字索引全局实例 """
